@@ -1,65 +1,122 @@
 /*
  * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2010-2019 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2019 Andy Green <andy@warmcat.com>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation:
- *  version 2.1 of the License.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA  02110-1301  USA
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
  */
 
-#include "core/private.h"
+#include "private-lib-core.h"
 
+#if defined(LWS_WITH_CLIENT)
+static int
+lws_close_trans_q_leader(struct lws_dll2 *d, void *user)
+{
+	struct lws *w = lws_container_of(d, struct lws, dll2_cli_txn_queue);
+
+	__lws_close_free_wsi(w, -1, "trans q leader closing");
+
+	return 0;
+}
+#endif
 
 void
-__lws_free_wsi(struct lws *wsi)
+__lws_reset_wsi(struct lws *wsi)
 {
 	if (!wsi)
 		return;
 
+#if defined(LWS_WITH_CLIENT)
+
+	lws_free_set_NULL(wsi->cli_hostname_copy);
+
+	/*
+	 * if we have wsi in our transaction queue, if we are closing we
+	 * must go through and close all those first
+	 */
+	if (wsi->vhost) {
+
+		/* we are no longer an active client connection that can piggyback */
+		lws_dll2_remove(&wsi->dll_cli_active_conns);
+
+		lws_dll2_foreach_safe(&wsi->dll2_cli_txn_queue_owner, NULL,
+				      lws_close_trans_q_leader);
+
+		/*
+		 * !!! If we are closing, but we have pending pipelined
+		 * transaction results we already sent headers for, that's going
+		 * to destroy sync for HTTP/1 and leave H2 stream with no live
+		 * swsi.`
+		 *
+		 * However this is normal if we are being closed because the
+		 * transaction queue leader is closing.
+		 */
+		lws_dll2_remove(&wsi->dll2_cli_txn_queue);
+	}
+#endif
+
+	if (wsi->vhost)
+		lws_dll2_remove(&wsi->vh_awaiting_socket);
+
 	/*
 	 * Protocol user data may be allocated either internally by lws
 	 * or by specified the user. We should only free what we allocated.
 	 */
 	if (wsi->protocol && wsi->protocol->per_session_data_size &&
 	    wsi->user_space && !wsi->user_space_externally_allocated)
-		lws_free(wsi->user_space);
+		lws_free_set_NULL(wsi->user_space);
 
 	lws_buflist_destroy_all_segments(&wsi->buflist);
 	lws_buflist_destroy_all_segments(&wsi->buflist_out);
+#if defined(LWS_WITH_UDP)
 	lws_free_set_NULL(wsi->udp);
+#endif
+	wsi->retry = 0;
+
+#if defined(LWS_WITH_CLIENT)
+	lws_dll2_remove(&wsi->dll2_cli_txn_queue);
+	lws_dll2_remove(&wsi->dll_cli_active_conns);
+#endif
+
+#if defined(LWS_WITH_SYS_ASYNC_DNS)
+	lws_async_dns_cancel(wsi);
+#endif
+
+#if defined(LWS_WITH_HTTP_PROXY)
+	if (wsi->http.buflist_post_body)
+		lws_buflist_destroy_all_segments(&wsi->http.buflist_post_body);
+#endif
 
 	if (wsi->vhost && wsi->vhost->lserv_wsi == wsi)
 		wsi->vhost->lserv_wsi = NULL;
-#if !defined(LWS_NO_CLIENT)
+#if defined(LWS_WITH_CLIENT)
 	if (wsi->vhost)
 		lws_dll2_remove(&wsi->dll_cli_active_conns);
 #endif
 	wsi->context->count_wsi_allocated--;
 
-#if defined(LWS_ROLE_H1) || defined(LWS_ROLE_H2)
-	__lws_header_table_detach(wsi, 0);
-#endif
 	__lws_same_vh_protocol_remove(wsi);
-#if !defined(LWS_NO_CLIENT)
-	lws_client_stash_destroy(wsi);
+#if defined(LWS_WITH_CLIENT)
+	lws_free_set_NULL(wsi->stash);
 	lws_free_set_NULL(wsi->cli_hostname_copy);
 #endif
 
-	if (wsi->role_ops->destroy_role)
-		wsi->role_ops->destroy_role(wsi);
-
 #if defined(LWS_WITH_PEER_LIMITS)
 	lws_peer_track_wsi_close(wsi->context, wsi->peer);
 	wsi->peer = NULL;
@@ -72,13 +129,30 @@ __lws_free_wsi(struct lws *wsi)
 #endif
 	__lws_wsi_remove_from_sul(wsi);
 
+	if (wsi->role_ops->destroy_role)
+		wsi->role_ops->destroy_role(wsi);
+
+#if defined(LWS_ROLE_H1) || defined(LWS_ROLE_H2)
+	__lws_header_table_detach(wsi, 0);
+#endif
+}
+
+void
+__lws_free_wsi(struct lws *wsi)
+{
+	if (!wsi)
+		return;
+
+	__lws_reset_wsi(wsi);
+
 	if (wsi->context->event_loop_ops->destroy_wsi)
 		wsi->context->event_loop_ops->destroy_wsi(wsi);
 
 	lws_vhost_unbind_wsi(wsi);
 
-	lwsl_debug("%s: %p, remaining wsi %d\n", __func__, wsi,
-			wsi->context->count_wsi_allocated);
+	lwsl_debug("%s: %p, remaining wsi %d, tsi fds count %d\n", __func__, wsi,
+			wsi->context->count_wsi_allocated,
+			wsi->context->pt[(int)wsi->tsi].fds_count);
 
 	lws_free(wsi);
 }
@@ -117,20 +191,12 @@ lws_remove_child_from_any_parent(struct lws *wsi)
 	wsi->parent = NULL;
 }
 
-#if !defined(LWS_NO_CLIENT)
-static int
-lws_close_trans_q_leader(struct lws_dll2 *d, void *user)
-{
-	struct lws *w = lws_container_of(d, struct lws, dll2_cli_txn_queue);
-
-	__lws_close_free_wsi(w, -1, "trans q leader closing");
-
-	return 0;
-}
-
+#if defined(LWS_WITH_CLIENT)
 void
 lws_inform_client_conn_fail(struct lws *wsi, void *arg, size_t len)
 {
+	lws_addrinfo_clean(wsi);
+
 	if (wsi->already_did_cce)
 		return;
 
@@ -141,23 +207,38 @@ lws_inform_client_conn_fail(struct lws *wsi, void *arg, size_t len)
 	if (!wsi->protocol)
 		return;
 
-	wsi->protocol->callback(wsi,
-			        LWS_CALLBACK_CLIENT_CONNECTION_ERROR,
-				wsi->user_space, arg, len);
+	if (!wsi->client_suppress_CONNECTION_ERROR)
+		wsi->protocol->callback(wsi,
+					LWS_CALLBACK_CLIENT_CONNECTION_ERROR,
+					wsi->user_space, arg, len);
 }
 #endif
 
+void
+lws_addrinfo_clean(struct lws *wsi)
+{
+#if defined(LWS_WITH_CLIENT)
+	if (!wsi->dns_results)
+		return;
+
+#if defined(LWS_WITH_SYS_ASYNC_DNS)
+	lws_async_dns_freeaddrinfo(&wsi->dns_results);
+#else
+	freeaddrinfo((struct addrinfo *)wsi->dns_results);
+#endif
+	wsi->dns_results = NULL;
+#endif
+}
+
 void
 __lws_close_free_wsi(struct lws *wsi, enum lws_close_status reason,
 		     const char *caller)
 {
 	struct lws_context_per_thread *pt;
-	struct lws *wsi1, *wsi2;
+	const struct lws_protocols *pro;
 	struct lws_context *context;
-#if !defined(LWS_NO_CLIENT)
-	long rl = (long)(int)reason;
-#endif
-	int n;
+	struct lws *wsi1, *wsi2;
+	int n, ccb;
 
 	lwsl_info("%s: %p: caller: %s\n", __func__, wsi, caller);
 
@@ -166,42 +247,24 @@ __lws_close_free_wsi(struct lws *wsi, enum lws_close_status reason,
 
 	lws_access_log(wsi);
 
+	if (!lws_dll2_is_detached(&wsi->dll_buflist)) {
+		lwsl_info("%s: wsi %p: going down with stuff in buflist\n",
+				__func__, wsi); }
+
 	context = wsi->context;
 	pt = &context->pt[(int)wsi->tsi];
 	lws_stats_bump(pt, LWSSTATS_C_API_CLOSE, 1);
 
-#if !defined(LWS_NO_CLIENT)
+#if defined(LWS_WITH_CLIENT)
 
 	lws_free_set_NULL(wsi->cli_hostname_copy);
 
-	/*
-	 * if we have wsi in our transaction queue, if we are closing we
-	 * must go through and close all those first
-	 */
-	if (wsi->vhost) {
+	lws_addrinfo_clean(wsi);
+#endif
 
-		/* we are no longer an active client connection that can piggyback */
-		lws_dll2_remove(&wsi->dll_cli_active_conns);
-
-		if (rl != -1l)
-			lws_vhost_lock(wsi->vhost);
-
-		lws_dll2_foreach_safe(&wsi->dll2_cli_txn_queue_owner, NULL,
-				      lws_close_trans_q_leader);
-
-		/*
-		 * !!! If we are closing, but we have pending pipelined
-		 * transaction results we already sent headers for, that's going
-		 * to destroy sync for HTTP/1 and leave H2 stream with no live
-		 * swsi.`
-		 *
-		 * However this is normal if we are being closed because the
-		 * transaction queue leader is closing.
-		 */
-		lws_dll2_remove(&wsi->dll2_cli_txn_queue);
-		if (rl != -1l)
-			lws_vhost_unlock(wsi->vhost);
-	}
+#if defined(LWS_WITH_HTTP2)
+	if (wsi->mux_stream_immortal)
+		lws_http_close_immortal(wsi);
 #endif
 
 	/* if we have children, close them first */
@@ -219,6 +282,7 @@ __lws_close_free_wsi(struct lws *wsi, enum lws_close_status reason,
 		wsi->child_list = NULL;
 	}
 
+#if defined(LWS_ROLE_RAW_FILE)
 	if (wsi->role_ops == &role_ops_raw_file) {
 		lws_remove_child_from_any_parent(wsi);
 		__remove_wsi_socket_from_fds(wsi);
@@ -227,13 +291,14 @@ __lws_close_free_wsi(struct lws *wsi, enum lws_close_status reason,
 					wsi->user_space, NULL, 0);
 		goto async_close;
 	}
+#endif
 
 	wsi->wsistate_pre_close = wsi->wsistate;
 
 #ifdef LWS_WITH_CGI
 	if (wsi->role_ops == &role_ops_cgi) {
 
-		// lwsl_debug("%s: closing stdwsi index %d\n", __func__, (int)wsi->cgi_channel);
+		// lwsl_debug("%s: closing stdwsi index %d\n", __func__, (int)wsi->lsp_channel);
 
 		/* we are not a network connection, but a handler for CGI io */
 		if (wsi->parent && wsi->parent->http.cgi) {
@@ -242,7 +307,8 @@ __lws_close_free_wsi(struct lws *wsi, enum lws_close_status reason,
 				lws_cgi_remove_and_kill(wsi->parent);
 
 			/* end the binding between us and master */
-			wsi->parent->http.cgi->stdwsi[(int)wsi->cgi_channel] =
+			if (wsi->parent->http.cgi)
+				wsi->parent->http.cgi->lsp->stdwsi[(int)wsi->lsp_channel] =
 									NULL;
 		}
 		wsi->socket_is_permanently_unusable = 1;
@@ -254,15 +320,15 @@ __lws_close_free_wsi(struct lws *wsi, enum lws_close_status reason,
 		lws_cgi_remove_and_kill(wsi);
 #endif
 
-#if !defined(LWS_NO_CLIENT)
-	lws_client_stash_destroy(wsi);
+#if defined(LWS_WITH_CLIENT)
+	lws_free_set_NULL(wsi->stash);
 #endif
 
 	if (wsi->role_ops == &role_ops_raw_skt) {
 		wsi->socket_is_permanently_unusable = 1;
 		goto just_kill_connection;
 	}
-#if defined(LWS_ROLE_H1) || defined(LWS_ROLE_H2)
+#if defined(LWS_WITH_FILE_OPS) && (defined(LWS_ROLE_H1) || defined(LWS_ROLE_H2))
 	if (lwsi_role_http(wsi) && lwsi_role_server(wsi) &&
 	    wsi->http.fop_fd != NULL)
 		lws_vfs_file_close(&wsi->http.fop_fd);
@@ -315,6 +381,7 @@ __lws_close_free_wsi(struct lws *wsi, enum lws_close_status reason,
 	}
 
 	if (lwsi_state(wsi) == LRS_WAITING_CONNECT ||
+	    lwsi_state(wsi) == LRS_WAITING_DNS ||
 	    lwsi_state(wsi) == LRS_H1C_ISSUE_HANDSHAKE)
 		goto just_kill_connection;
 
@@ -346,15 +413,23 @@ __lws_close_free_wsi(struct lws *wsi, enum lws_close_status reason,
 just_kill_connection:
 
 #if defined(LWS_WITH_FILE_OPS) && (defined(LWS_ROLE_H1) || defined(LWS_ROLE_H2))
-       if (lwsi_role_http(wsi) && lwsi_role_server(wsi) &&
-           wsi->http.fop_fd != NULL)
-	       lws_vfs_file_close(&wsi->http.fop_fd);
+	if (lwsi_role_http(wsi) && lwsi_role_server(wsi) &&
+	    wsi->http.fop_fd != NULL)
+		lws_vfs_file_close(&wsi->http.fop_fd);
+#endif
+
+#if defined(LWS_WITH_SYS_ASYNC_DNS)
+	lws_async_dns_cancel(wsi);
 #endif
 
 #if defined(LWS_WITH_HTTP_PROXY)
 	if (wsi->http.buflist_post_body)
 		lws_buflist_destroy_all_segments(&wsi->http.buflist_post_body);
 #endif
+#if defined(LWS_WITH_UDP)
+	if (wsi->udp)
+		lws_free_set_NULL(wsi->udp);
+#endif
 
 	if (wsi->role_ops->close_kill_connection)
 		wsi->role_ops->close_kill_connection(wsi, reason);
@@ -373,8 +448,9 @@ just_kill_connection:
 		wsi->protocol_bind_balance = 0;
 	}
 
-#if !defined(LWS_NO_CLIENT)
+#if defined(LWS_WITH_CLIENT)
 	if ((lwsi_state(wsi) == LRS_WAITING_SERVER_REPLY ||
+	     lwsi_state(wsi) == LRS_WAITING_DNS ||
 	     lwsi_state(wsi) == LRS_WAITING_CONNECT) &&
 	     !wsi->already_did_cce && wsi->protocol) {
 		static const char _reason[] = "closed before established";
@@ -412,7 +488,7 @@ just_kill_connection:
 #endif
 		{
 			lwsl_info("%s: shutdown conn: %p (sk %d, state 0x%x)\n",
-				  __func__, wsi, (int)(long)wsi->desc.sockfd,
+				  __func__, wsi, (int)(lws_intptr_t)wsi->desc.sockfd,
 				  lwsi_state(wsi));
 			if (!wsi->socket_is_permanently_unusable &&
 			    lws_socket_is_valid(wsi->desc.sockfd)) {
@@ -428,12 +504,12 @@ just_kill_connection:
 		 * This causes problems on WINCE / ESP32 with disconnection
 		 * when the events are half closing connection
 		 */
-#if !defined(_WIN32_WCE) && !defined(LWS_WITH_ESP32)
+#if !defined(_WIN32_WCE) && !defined(LWS_PLAT_FREERTOS)
 		/* libuv: no event available to guarantee completion */
 		if (!wsi->socket_is_permanently_unusable &&
 		    lws_socket_is_valid(wsi->desc.sockfd) &&
 		    lwsi_state(wsi) != LRS_SHUTDOWN &&
-		    context->event_loop_ops->periodic_events_available) {
+		    (context->event_loop_ops->flags & LELOF_ISPOLL)) {
 			__lws_change_pollfd(wsi, LWS_POLLOUT, LWS_POLLIN);
 			lwsi_set_state(wsi, LRS_SHUTDOWN);
 			__lws_set_timeout(wsi, PENDING_TIMEOUT_SHUTDOWN_FLUSH,
@@ -484,18 +560,15 @@ just_kill_connection:
 
 	/* tell the user it's all over for this guy */
 
+	ccb = 0;
 	if ((lwsi_state_est_PRE_CLOSE(wsi) ||
 	    /* raw skt adopted but didn't complete tls hs should CLOSE */
 	    (wsi->role_ops == &role_ops_raw_skt && !lwsi_role_client(wsi)) ||
 	     lwsi_state_PRE_CLOSE(wsi) == LRS_WAITING_SERVER_REPLY) &&
 	    !wsi->told_user_closed &&
 	    wsi->role_ops->close_cb[lwsi_role_server(wsi)]) {
-		const struct lws_protocols *pro = wsi->protocol;
-
-		if (!wsi->protocol && wsi->vhost && wsi->vhost->protocols)
-			pro = &wsi->vhost->protocols[0];
-
-		if (pro && (!wsi->upgraded_to_http2 || !lwsi_role_client(wsi)))
+		if (!wsi->upgraded_to_http2 || !lwsi_role_client(wsi))
+			ccb = 1;
 			/*
 			 * The network wsi for a client h2 connection shouldn't
 			 * call back for its role: the child stream connections
@@ -503,13 +576,40 @@ just_kill_connection:
 			 * one too many times as the children do it and then
 			 * the closing network stream.
 			 */
+	}
+
+	if (!wsi->told_user_closed &&
+	    !lws_dll2_is_detached(&wsi->vh_awaiting_socket))
+		/*
+		 * He's a guy who go started with dns, but failed or is
+		 * caught with a shutdown before he got the result.  We have
+		 * to issue him a close cb
+		 */
+		ccb = 1;
+
+	pro = wsi->protocol;
+
+#if defined(LWS_WITH_CLIENT)
+	if (!ccb && (lwsi_state_PRE_CLOSE(wsi) & LWSIFS_NOT_EST) &&
+			lwsi_role_client(wsi)) {
+		lws_inform_client_conn_fail(wsi, "Closed before conn", 18);
+	}
+#endif
+	if (ccb) {
+
+		if (!wsi->protocol && wsi->vhost && wsi->vhost->protocols)
+			pro = &wsi->vhost->protocols[0];
+
+		if (pro)
 			pro->callback(wsi,
-			      wsi->role_ops->close_cb[lwsi_role_server(wsi)],
-			      wsi->user_space, NULL, 0);
+				wsi->role_ops->close_cb[lwsi_role_server(wsi)],
+				wsi->user_space, NULL, 0);
 		wsi->told_user_closed = 1;
 	}
 
+#if defined(LWS_ROLE_RAW_FILE)
 async_close:
+#endif
 	lws_remove_child_from_any_parent(wsi);
 	wsi->socket_is_permanently_unusable = 1;
 
@@ -542,17 +642,7 @@ __lws_close_free_wsi_final(struct lws *wsi)
 
 #ifdef LWS_WITH_CGI
 	if (wsi->http.cgi) {
-
-		for (n = 0; n < 3; n++) {
-			if (wsi->http.cgi->pipe_fds[n][!!(n == 0)] == 0)
-				lwsl_err("ZERO FD IN CGI CLOSE");
-
-			if (wsi->http.cgi->pipe_fds[n][!!(n == 0)] >= 0) {
-				close(wsi->http.cgi->pipe_fds[n][!!(n == 0)]);
-				wsi->http.cgi->pipe_fds[n][!!(n == 0)] = LWS_SOCK_INVALID;
-			}
-		}
-
+		lws_spawn_piped_destroy(&wsi->http.cgi->lsp);
 		lws_free_set_NULL(wsi->http.cgi);
 	}
 #endif
