@@ -1,54 +1,40 @@
 /*
  * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2010-2018 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2019 Andy Green <andy@warmcat.com>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation:
- *  version 2.1 of the License.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA  02110-1301  USA
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
  */
 
-#include "core/private.h"
+#include <libwebsockets.h>
+#include "private-lib-core.h"
 
-void
-lws_client_stash_destroy(struct lws *wsi)
-{
-	if (!wsi || !wsi->stash)
-		return;
-
-	lws_free_set_NULL(wsi->stash->address);
-	lws_free_set_NULL(wsi->stash->path);
-	lws_free_set_NULL(wsi->stash->host);
-	lws_free_set_NULL(wsi->stash->origin);
-	lws_free_set_NULL(wsi->stash->protocol);
-	lws_free_set_NULL(wsi->stash->method);
-	lws_free_set_NULL(wsi->stash->iface);
-	lws_free_set_NULL(wsi->stash->alpn);
-
-	lws_free_set_NULL(wsi->stash);
-}
-
-LWS_VISIBLE struct lws *
+struct lws *
 lws_client_connect_via_info(const struct lws_client_connect_info *i)
 {
+	const char *local = i->protocol;
 	struct lws *wsi, *safe = NULL;
 	const struct lws_protocols *p;
-	const char *local = i->protocol;
-	int tid = 0;
-#if LWS_MAX_SMP > 1
-	int n;
-#endif
+	const char *cisin[CIS_COUNT];
+	int tid = 0, n, m;
+	size_t size;
+	char *pc;
 
 	if (i->context->requested_kill)
 		return NULL;
@@ -72,14 +58,33 @@ lws_client_connect_via_info(const struct lws_client_connect_info *i)
 	if (wsi == NULL)
 		goto bail;
 
+	/*
+	 * Until we exit, we can report connection failure directly to the
+	 * caller without needing to call through to protocol CONNECTION_ERROR.
+	 */
+	wsi->client_suppress_CONNECTION_ERROR = 1;
+
 	wsi->context = i->context;
 	wsi->desc.sockfd = LWS_SOCK_INVALID;
 	wsi->seq = i->seq;
+	wsi->flags = i->ssl_connection;
+	if (i->retry_and_idle_policy)
+		wsi->retry_policy = i->retry_and_idle_policy;
+	else
+		wsi->retry_policy = &i->context->default_retry;
+
+#if defined(LWS_WITH_DETAILED_LATENCY)
+	if (i->context->detailed_latency_cb)
+		wsi->detlat.earliest_write_req_pre_write = lws_now_usecs();
+#endif
 
 	wsi->vhost = NULL;
-	if (!i->vhost)
-		lws_vhost_bind_wsi(i->context->vhost_list, wsi);
-	else
+	if (!i->vhost) {
+		struct lws_vhost *v = i->context->vhost_list;
+		if (v && !strcmp(v->name, "system"))
+			v = v->vhost_next;
+		lws_vhost_bind_wsi(v, wsi);
+	} else
 		lws_vhost_bind_wsi(i->vhost, wsi);
 
 	if (!wsi->vhost) {
@@ -106,6 +111,9 @@ lws_client_connect_via_info(const struct lws_client_connect_info *i)
 			lwsl_info("%s: client binds to caller tsi %d\n",
 				  __func__, n);
 			wsi->tsi = n;
+#if defined(LWS_WITH_DETAILED_LATENCY)
+			wsi->detlat.tsi = n;
+#endif
 			break;
 		}
 
@@ -142,16 +150,23 @@ lws_client_connect_via_info(const struct lws_client_connect_info *i)
 	wsi->pending_timeout = NO_PENDING_TIMEOUT;
 	wsi->position_in_fds_table = LWS_NO_FDS_POS;
 	wsi->ocport = wsi->c_port = i->port;
+	wsi->sys_tls_client_cert = i->sys_tls_client_cert;
+
+#if defined(LWS_ROLE_H2)
+	wsi->txc.manual_initial_tx_credit = (int32_t)i->manual_initial_tx_credit;
+#endif
 
 	wsi->protocol = &wsi->vhost->protocols[0];
 	wsi->client_pipeline = !!(i->ssl_connection & LCCSCF_PIPELINE);
+	wsi->client_no_follow_redirect = !!(i->ssl_connection &
+					    LCCSCF_HTTP_NO_FOLLOW_REDIRECT);
 
 	/*
 	 * PHASE 5: handle external user_space now, generic alloc is done in
 	 * role finalization
 	 */
 
-	if (!wsi->user_space && i->userdata) {
+	if (i->userdata) {
 		wsi->user_space_externally_allocated = 1;
 		wsi->user_space = i->userdata;
 	}
@@ -161,6 +176,12 @@ lws_client_connect_via_info(const struct lws_client_connect_info *i)
 		p = lws_vhost_name_to_protocol(wsi->vhost, local);
 		if (p)
 			lws_bind_protocol(wsi, p, __func__);
+		else
+			lwsl_err("%s: unknown protocol %s\n", __func__, local);
+
+		lwsl_info("%s: wsi %p: %s %s entry\n",
+			    __func__, wsi, wsi->role_ops->name,
+			    wsi->protocol ? wsi->protocol->name : "none");
 	}
 
 	/*
@@ -193,45 +214,44 @@ lws_client_connect_via_info(const struct lws_client_connect_info *i)
 	 * with no relationship to http or ah
 	 */
 
-	wsi->stash = lws_zalloc(sizeof(*wsi->stash), "client stash");
+	cisin[CIS_ADDRESS]	= i->address;
+	cisin[CIS_PATH]		= i->path;
+	cisin[CIS_HOST]		= i->host;
+	cisin[CIS_ORIGIN]	= i->origin;
+	cisin[CIS_PROTOCOL]	= i->protocol;
+	cisin[CIS_METHOD]	= i->method;
+	cisin[CIS_IFACE]	= i->iface;
+	cisin[CIS_ALPN]		= i->alpn;
+
+	size = sizeof(*wsi->stash);
+
+	/*
+	 * Let's overallocate the stash object with space for all the args
+	 * in one hit.
+	 */
+	for (n = 0; n < CIS_COUNT; n++)
+		if (cisin[n])
+			size += strlen(cisin[n]) + 1;
+
+	wsi->stash = lws_malloc(size, "client stash");
 	if (!wsi->stash) {
 		lwsl_err("%s: OOM\n", __func__);
 		goto bail1;
 	}
+	/* all the pointers default to NULL, but no need to zero the args */
+	memset(wsi->stash, 0, sizeof(*wsi->stash));
 
-	wsi->stash->address = lws_strdup(i->address);
-	wsi->stash->path = lws_strdup(i->path);
-	wsi->stash->host = lws_strdup(i->host);
-	wsi->stash->opaque_user_data = i->opaque_user_data;
+	wsi->opaque_user_data = wsi->stash->opaque_user_data =
+		i->opaque_user_data;
+	pc = (char *)&wsi->stash[1];
 
-	if (!wsi->stash->address || !wsi->stash->path || !wsi->stash->host)
-		goto bail1;
-
-	if (i->origin) {
-		wsi->stash->origin = lws_strdup(i->origin);
-		if (!wsi->stash->origin)
-			goto bail1;
-	}
-	if (i->protocol) {
-		wsi->stash->protocol = lws_strdup(i->protocol);
-		if (!wsi->stash->protocol)
-			goto bail1;
-	}
-	if (i->method) {
-		wsi->stash->method = lws_strdup(i->method);
-		if (!wsi->stash->method)
-			goto bail1;
-	}
-	if (i->iface) {
-		wsi->stash->iface = lws_strdup(i->iface);
-		if (!wsi->stash->iface)
-			goto bail1;
-	}
-	if (i->alpn) {
-		wsi->stash->alpn = lws_strdup(i->alpn);
-		if (!wsi->stash->alpn)
-			goto bail1;
-	}
+	for (n = 0; n < CIS_COUNT; n++)
+		if (cisin[n]) {
+			wsi->stash->cis[n] = pc;
+			m = (int)strlen(cisin[n]) + 1;
+			memcpy(pc, cisin[n], m);
+			pc += m;
+		}
 
 	/*
 	 * at this point user callbacks like
@@ -278,13 +298,18 @@ lws_client_connect_via_info(const struct lws_client_connect_info *i)
 
 	/* PHASE 8: notify protocol with role-specific connected callback */
 
-	lwsl_debug("%s: wsi %p: cb %d to %s %s\n", __func__,
-			wsi, wsi->role_ops->adoption_cb[0],
-			wsi->role_ops->name, wsi->protocol->name);
+	/* raw socket per se doesn't want this... raw socket proxy wants it... */
 
-	wsi->protocol->callback(wsi,
-			wsi->role_ops->adoption_cb[0],
-			wsi->user_space, NULL, 0);
+	if (wsi->role_ops != &role_ops_raw_skt ||
+	    (i->local_protocol_name &&
+	     !strcmp(i->local_protocol_name, "raw-proxy"))) {
+		lwsl_debug("%s: wsi %p: adoption cb %d to %s %s\n", __func__,
+			   wsi, wsi->role_ops->adoption_cb[0],
+			   wsi->role_ops->name, wsi->protocol->name);
+
+		wsi->protocol->callback(wsi, wsi->role_ops->adoption_cb[0],
+				wsi->user_space, NULL, 0);
+	}
 
 #if defined(LWS_WITH_HUBBUB)
 	if (i->uri_replace_to)
@@ -293,19 +318,72 @@ lws_client_connect_via_info(const struct lws_client_connect_info *i)
 					     i->uri_replace_to);
 #endif
 
-	if (i->method && !strcmp(i->method, "RAW"))
-		lws_http_client_connect_via_info2(wsi);
+	if (i->method && (!strcmp(i->method, "RAW") // ||
+//			  !strcmp(i->method, "MQTT")
+	)) {
+
+		/*
+		 * Not for MQTT here, since we don't know if we will
+		 * pipeline it or not...
+		 */
+
+#if defined(LWS_WITH_TLS)
+
+		wsi->tls.ssl = NULL;
+
+		if (wsi->tls.use_ssl & LCCSCF_USE_SSL) {
+			const char *cce = NULL;
+
+			switch (
+#if !defined(LWS_WITH_SYS_ASYNC_DNS)
+			lws_client_create_tls(wsi, &cce, 1)
+#else
+			lws_client_create_tls(wsi, &cce, 0)
+#endif
+			) {
+			case 1:
+				return wsi;
+			case 0:
+				break;
+			default:
+				goto bail3;
+			}
+		}
+#endif
+
+		/* fallthru */
+
+		wsi = lws_http_client_connect_via_info2(wsi);
+	}
+
+	if (wsi)
+		/*
+		 * If it subsequently fails, report CONNECTION_ERROR,
+		 * because we're going to return a non-error return now.
+		 */
+		wsi->client_suppress_CONNECTION_ERROR = 0;
 
 	return wsi;
 
+#if defined(LWS_WITH_TLS)
+bail3:
+	lws_close_free_wsi(wsi, LWS_CLOSE_STATUS_NOSTATUS, "tls start fail");
+
+	return NULL;
+#endif
+
 bail1:
-	lws_client_stash_destroy(wsi);
+	lws_free_set_NULL(wsi->stash);
 
 bail:
 	lws_free(wsi);
 #if defined(LWS_ROLE_H1) || defined(LWS_ROLE_H2)
 bail2:
 #endif
+
+	if (i->ssl_connection & LCCSCF_USE_SSL)
+		lws_tls_restrict_return(i->context);
+
 	if (i->pwsi)
 		*i->pwsi = NULL;
 
