@@ -1,25 +1,28 @@
 /*
  * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2010-2018 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2019 Andy Green <andy@warmcat.com>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation:
- *  version 2.1 of the License.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA  02110-1301  USA
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
  */
 
-#include "core/private.h"
+#include "private-lib-core.h"
 
 #if defined(LWS_WITH_HTTP_PROXY)
 static int
@@ -54,7 +57,7 @@ stream_close(struct lws *wsi)
 
 	wsi->http.did_stream_close = 1;
 
-	if (wsi->http2_substream) {
+	if (wsi->mux_substream) {
 		if (lws_write(wsi, (unsigned char *)buf + LWS_PRE, 0,
 			      LWS_WRITE_HTTP_FINAL) < 0) {
 			lwsl_info("%s: COMPL_CLIENT_HTTP: h2 fin wr failed\n",
@@ -112,7 +115,7 @@ lws_callback_ws_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 		lws_process_ws_upgrade2(wsi->parent);
 
 #if defined(LWS_WITH_HTTP2)
-		if (wsi->parent->http2_substream)
+		if (wsi->parent->mux_substream)
 			lwsl_info("%s: proxied h2 -> h1 ws established\n", __func__);
 #endif
 		break;
@@ -124,9 +127,7 @@ lws_callback_ws_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 	case LWS_CALLBACK_CLIENT_CLOSED:
 		lwsl_user("%s: client closed: parent %p\n", __func__, wsi->parent);
 		if (wsi->parent)
-			lws_set_timeout(wsi->parent,
-				PENDING_TIMEOUT_KILLED_BY_PROXY_CLIENT_CLOSE,
-				LWS_TO_KILL_ASYNC);
+                       lws_set_timeout(wsi->parent, 1, LWS_TO_KILL_ASYNC);
 		break;
 
 	case LWS_CALLBACK_CLIENT_APPEND_HANDSHAKE_HEADER:
@@ -156,7 +157,6 @@ lws_callback_ws_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 		if (!pkt)
 			return -1;
 
-		pkt->pkt_list.prev = pkt->pkt_list.next = NULL;
 		pkt->len = len;
 		pkt->first = lws_is_first_fragment(wsi);
 		pkt->final = lws_is_final_fragment(wsi);
@@ -169,7 +169,7 @@ lws_callback_ws_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 		break;
 
 	case LWS_CALLBACK_CLIENT_WRITEABLE:
-		dll = lws_dll2_get_tail(&wsi->ws->proxy_owner);
+		dll = lws_dll2_get_head(&wsi->ws->proxy_owner);
 		if (!dll)
 			break;
 
@@ -180,12 +180,10 @@ lws_callback_ws_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 					pkt->first, pkt->final)) < 0)
 			return -1;
 
-		wsi->parent->ws->proxy_buffered -= pkt->len;
-
 		lws_dll2_remove(dll);
 		lws_free(pkt);
 
-		if (lws_dll2_get_tail(&wsi->ws->proxy_owner))
+		if (lws_dll2_get_head(&wsi->ws->proxy_owner))
 			lws_callback_on_writable(wsi);
 		break;
 
@@ -203,7 +201,6 @@ lws_callback_ws_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 		if (!pkt)
 			return -1;
 
-		pkt->pkt_list.prev = pkt->pkt_list.next = NULL;
 		pkt->len = len;
 		pkt->first = lws_is_first_fragment(wsi);
 		pkt->final = lws_is_final_fragment(wsi);
@@ -216,7 +213,7 @@ lws_callback_ws_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 		break;
 
 	case LWS_CALLBACK_SERVER_WRITEABLE:
-		dll = lws_dll2_get_tail(&wsi->ws->proxy_owner);
+		dll = lws_dll2_get_head(&wsi->ws->proxy_owner);
 		if (!dll)
 			break;
 
@@ -227,10 +224,12 @@ lws_callback_ws_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 					pkt->first, pkt->final)) < 0)
 			return -1;
 
+		wsi->ws->proxy_buffered -= pkt->len;
+
 		lws_dll2_remove(dll);
 		lws_free(pkt);
 
-		if (lws_dll2_get_tail(&wsi->ws->proxy_owner))
+		if (lws_dll2_get_head(&wsi->ws->proxy_owner))
 			lws_callback_on_writable(wsi);
 		break;
 
@@ -251,7 +250,7 @@ const struct lws_protocols lws_ws_proxy = {
 
 #endif
 
-LWS_VISIBLE int
+int
 lws_callback_http_dummy(struct lws *wsi, enum lws_callback_reasons reason,
 			void *user, void *in, size_t len)
 {
@@ -260,7 +259,7 @@ lws_callback_http_dummy(struct lws *wsi, enum lws_callback_reasons reason,
 	struct lws_cgi_args *args;
 #endif
 #if defined(LWS_WITH_CGI) || defined(LWS_WITH_HTTP_PROXY)
-	char buf[8192];
+	char buf[LWS_PRE + 32 + 8192];
 	int n;
 #endif
 #if defined(LWS_WITH_HTTP_PROXY)
@@ -271,7 +270,7 @@ lws_callback_http_dummy(struct lws *wsi, enum lws_callback_reasons reason,
 	switch (reason) {
 #if defined(LWS_ROLE_H1) || defined(LWS_ROLE_H2)
 	case LWS_CALLBACK_HTTP:
-#ifndef LWS_NO_SERVER
+#if defined(LWS_WITH_SERVER)
 		if (lws_return_http_status(wsi, HTTP_STATUS_NOT_FOUND, NULL))
 			return -1;
 
@@ -279,7 +278,7 @@ lws_callback_http_dummy(struct lws *wsi, enum lws_callback_reasons reason,
 #endif
 			return -1;
 		break;
-#if !defined(LWS_NO_SERVER)
+#if defined(LWS_WITH_SERVER)
 	case LWS_CALLBACK_HTTP_BODY_COMPLETION:
 #if defined(LWS_WITH_HTTP_PROXY)
 		if (wsi->child_list) {
@@ -315,9 +314,10 @@ lws_callback_http_dummy(struct lws *wsi, enum lws_callback_reasons reason,
 				lwsl_debug("AUX_BF__CGI forcing close\n");
 				return -1;
 			}
-			if (!n && wsi->http.cgi && wsi->http.cgi->stdwsi[LWS_STDOUT])
+			if (!n && wsi->http.cgi &&
+			    wsi->http.cgi->lsp->stdwsi[LWS_STDOUT])
 				lws_rx_flow_control(
-					wsi->http.cgi->stdwsi[LWS_STDOUT], 1);
+					wsi->http.cgi->lsp->stdwsi[LWS_STDOUT], 1);
 
 			if (wsi->reason_bf & LWS_CB_REASON_AUX_BF__CGI_HEADERS)
 				wsi->reason_bf &=
@@ -331,13 +331,13 @@ lws_callback_http_dummy(struct lws *wsi, enum lws_callback_reasons reason,
 		}
 
 		if (wsi->reason_bf & LWS_CB_REASON_AUX_BF__CGI_CHUNK_END) {
-			if (!wsi->http2_substream) {
+			if (!wsi->mux_substream) {
 				memcpy(buf + LWS_PRE, "0\x0d\x0a\x0d\x0a", 5);
 				lwsl_debug("writing chunk term and exiting\n");
-				n = lws_write(wsi, (unsigned char *)buf +
+				lws_write(wsi, (unsigned char *)buf +
 						   LWS_PRE, 5, LWS_WRITE_HTTP);
 			} else
-				n = lws_write(wsi, (unsigned char *)buf +
+				lws_write(wsi, (unsigned char *)buf +
 						   LWS_PRE, 0,
 						   LWS_WRITE_HTTP_FINAL);
 
@@ -509,7 +509,7 @@ lws_callback_http_dummy(struct lws *wsi, enum lws_callback_reasons reason,
 		proxy_header(parent, wsi, end, 256,
 			     WSI_TOKEN_HTTP_LOCATION, &p, end);
 
-		if (!parent->http2_substream)
+		if (!parent->mux_substream)
 			if (lws_add_http_header_by_token(parent,
 				WSI_TOKEN_CONNECTION, (unsigned char *)"close",
 				5, &p, end))
@@ -523,7 +523,7 @@ lws_callback_http_dummy(struct lws *wsi, enum lws_callback_reasons reason,
 		 * our own chunking since we still don't know the size.
 		 */
 
-		if (!parent->http2_substream &&
+		if (!parent->mux_substream &&
 		    !lws_hdr_total_length(wsi, WSI_TOKEN_HTTP_CONTENT_LENGTH)) {
 			lwsl_debug("downstream parent chunked\n");
 			if (lws_add_http_header_by_token(parent,
@@ -583,10 +583,9 @@ lws_callback_http_dummy(struct lws *wsi, enum lws_callback_reasons reason,
 	case LWS_CALLBACK_CLOSED_CLIENT_HTTP:
 		if (!lws_get_parent(wsi))
 			break;
-		lwsl_err("%s: LWS_CALLBACK_CLOSED_CLIENT_HTTP\n", __func__);
-		lws_set_timeout(lws_get_parent(wsi),
-				PENDING_TIMEOUT_KILLED_BY_PROXY_CLIENT_CLOSE,
-				LWS_TO_KILL_ASYNC);
+	//	lwsl_err("%s: LWS_CALLBACK_CLOSED_CLIENT_HTTP\n", __func__);
+               lws_set_timeout(lws_get_parent(wsi), LWS_TO_KILL_ASYNC,
+                               PENDING_TIMEOUT_KILLED_BY_PROXY_CLIENT_CLOSE);
 		break;
 
 	case LWS_CALLBACK_CLIENT_APPEND_HANDSHAKE_HEADER:
@@ -663,7 +662,7 @@ lws_callback_http_dummy(struct lws *wsi, enum lws_callback_reasons reason,
 		lwsl_debug("LWS_CALLBACK_CGI_TERMINATED: %d %" PRIu64 "\n",
 				wsi->http.cgi->explicitly_chunked,
 				(uint64_t)wsi->http.cgi->content_length);
-		if (!wsi->http.cgi->explicitly_chunked &&
+		if (!(wsi->http.cgi->explicitly_chunked && wsi->mux_substream) &&
 		    !wsi->http.cgi->content_length) {
 			/* send terminating chunk */
 			lwsl_debug("LWS_CALLBACK_CGI_TERMINATED: ending\n");
@@ -672,6 +671,10 @@ lws_callback_http_dummy(struct lws *wsi, enum lws_callback_reasons reason,
 			lws_set_timeout(wsi, PENDING_TIMEOUT_CGI, 3);
 			break;
 		}
+		if (wsi->mux_substream && !wsi->cgi_stdout_zero_length)
+			lws_write(wsi, (unsigned char *)buf + LWS_PRE, 0,
+						      LWS_WRITE_HTTP_FINAL);
+
 		if (lws_http_transaction_completed(wsi))
 			return -1;
 		return 0;
@@ -794,7 +797,7 @@ lws_callback_http_dummy(struct lws *wsi, enum lws_callback_reasons reason,
 					compatible_close(siwsi->desc.sockfd);
 					__lws_free_wsi(siwsi);
 				}
-				wsi->http.cgi->pipe_fds[LWS_STDIN][1] = -1;
+				wsi->http.cgi->lsp->pipe_fds[LWS_STDIN][1] = -1;
 
 //				args->stdwsi[LWS_STDIN] = NULL;
 			}
@@ -813,7 +816,7 @@ lws_callback_http_dummy(struct lws *wsi, enum lws_callback_reasons reason,
 
 #if LWS_MAX_SMP > 1
 	case LWS_CALLBACK_GET_THREAD_ID:
-		return (int)(unsigned long long)pthread_self();
+		return (int)(lws_intptr_t)pthread_self();
 #endif
 
 	default:
