@@ -1,30 +1,34 @@
 /*
  * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2010-2017 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2020 Andy Green <andy@warmcat.com>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation:
- *  version 2.1 of the License.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA  02110-1301  USA
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
  */
 
-#include "core/private.h"
+#include "private-lib-core.h"
 
 int
 _lws_change_pollfd(struct lws *wsi, int _and, int _or, struct lws_pollargs *pa)
 {
-#if !defined(LWS_WITH_LIBUV) && !defined(LWS_WITH_LIBEV) && !defined(LWS_WITH_LIBEVENT)
+#if !defined(LWS_WITH_LIBUV) && !defined(LWS_WITH_LIBEV) && \
+    !defined(LWS_WITH_LIBEVENT) && !defined(LWS_WITH_GLIB)
 	volatile struct lws_context_per_thread *vpt;
 #endif
 	struct lws_context_per_thread *pt;
@@ -68,9 +72,8 @@ _lws_change_pollfd(struct lws *wsi, int _and, int _or, struct lws_pollargs *pa)
 
 	assert(wsi->position_in_fds_table < (int)pt->fds_count);
 
-#if !defined(LWS_WITH_LIBUV) && \
-    !defined(LWS_WITH_LIBEV) && \
-    !defined(LWS_WITH_LIBEVENT)
+#if !defined(LWS_WITH_LIBUV) && !defined(LWS_WITH_LIBEV) && \
+    !defined(LWS_WITH_LIBEVENT) && !defined(LWS_WITH_GLIB)
 	/*
 	 * This only applies when we use the default poll() event loop.
 	 *
@@ -137,6 +140,11 @@ _lws_change_pollfd(struct lws *wsi, int _and, int _or, struct lws_pollargs *pa)
 	lws_memory_barrier();
 #endif
 
+#if !defined(__linux__)
+	/* OSX couldn't see close on stdin pipe side otherwise */
+	_or |= LWS_POLLHUP;
+#endif
+
 	pfd = &pt->fds[wsi->position_in_fds_table];
 	pa->fd = wsi->desc.sockfd;
 	lwsl_debug("%s: wsi %p: fd %d events %d -> %d\n", __func__, wsi,
@@ -144,7 +152,7 @@ _lws_change_pollfd(struct lws *wsi, int _and, int _or, struct lws_pollargs *pa)
 	pa->prev_events = pfd->events;
 	pa->events = pfd->events = (pfd->events & ~_and) | _or;
 
-	if (wsi->http2_substream)
+	if (wsi->mux_substream)
 		return 0;
 
 #if defined(LWS_WITH_EXTERNAL_POLL)
@@ -208,7 +216,7 @@ bail:
 	return ret;
 }
 
-#ifndef LWS_NO_SERVER
+#if defined(LWS_WITH_SERVER)
 /*
  * Enable or disable listen sockets on this pt globally...
  * it's modulated according to the pt having space for a new accept.
@@ -234,7 +242,7 @@ lws_accept_modulation(struct lws_context *context,
 }
 #endif
 
-#if defined(_DEBUG)
+#if _LWS_ENABLED_LOGS & LLL_WARN
 void
 __dump_fds(struct lws_context_per_thread *pt, const char *s)
 {
@@ -317,8 +325,8 @@ __insert_wsi_socket_into_fds(struct lws_context *context, struct lws *wsi)
 					   wsi->user_space, (void *) &pa, 0))
 		ret =  -1;
 #endif
-#ifndef LWS_NO_SERVER
-	/* if no more room, defeat accepts on this thread */
+#if defined(LWS_WITH_SERVER)
+	/* if no more room, defeat accepts on this service thread */
 	if ((unsigned int)pt->fds_count == context->fd_limit_per_thread - 1)
 		lws_accept_modulation(context, pt, 0);
 #endif
@@ -427,7 +435,7 @@ __remove_wsi_socket_from_fds(struct lws *wsi)
 		ret = -1;
 #endif
 
-#ifndef LWS_NO_SERVER
+#if defined(LWS_WITH_SERVER)
 	if (!context->being_destroyed &&
 	    /* if this made some room, accept connects on this thread */
 	    (unsigned int)pt->fds_count < context->fd_limit_per_thread - 1)
@@ -495,10 +503,11 @@ lws_change_pollfd(struct lws *wsi, int _and, int _or)
 	return ret;
 }
 
-LWS_VISIBLE int
+int
 lws_callback_on_writable(struct lws *wsi)
 {
 	struct lws_context_per_thread *pt;
+	struct lws *w = wsi;
 
 	if (lwsi_state(wsi) == LRS_SHUTDOWN)
 		return 0;
@@ -508,6 +517,11 @@ lws_callback_on_writable(struct lws *wsi)
 
 	pt = &wsi->context->pt[(int)wsi->tsi];
 
+#if defined(LWS_WITH_DETAILED_LATENCY)
+	if (!wsi->detlat.earliest_write_req)
+		wsi->detlat.earliest_write_req = lws_now_usecs();
+#endif
+
 	lws_stats_bump(pt, LWSSTATS_C_WRITEABLE_CB_REQ, 1);
 #if defined(LWS_WITH_STATS)
 	if (!wsi->active_writable_req_us) {
@@ -516,20 +530,23 @@ lws_callback_on_writable(struct lws *wsi)
 	}
 #endif
 
-
 	if (wsi->role_ops->callback_on_writable) {
-		if (wsi->role_ops->callback_on_writable(wsi))
+		int q = wsi->role_ops->callback_on_writable(wsi);
+		//lwsl_notice("%s: rops_cow says %d\n", __func__, q);
+		if (q)
 			return 1;
-		wsi = lws_get_network_wsi(wsi);
-	}
+		w = lws_get_network_wsi(wsi);
+	} else
 
-	if (wsi->position_in_fds_table == LWS_NO_FDS_POS) {
-		lwsl_debug("%s: failed to find socket %d\n", __func__,
-			   wsi->desc.sockfd);
-		return -1;
-	}
+		if (w->position_in_fds_table == LWS_NO_FDS_POS) {
+			lwsl_debug("%s: failed to find socket %d\n", __func__,
+				   wsi->desc.sockfd);
+			return -1;
+		}
 
-	if (__lws_change_pollfd(wsi, 0, LWS_POLLOUT))
+	//lwsl_notice("%s: marking for POLLOUT %p (wsi %p)\n", __func__, w, wsi);
+
+	if (__lws_change_pollfd(w, 0, LWS_POLLOUT))
 		return -1;
 
 	return 1;
@@ -580,7 +597,7 @@ lws_same_vh_protocol_remove(struct lws *wsi)
 }
 
 
-LWS_VISIBLE int
+int
 lws_callback_on_writable_all_protocol_vhost(const struct lws_vhost *vhost,
 				           const struct lws_protocols *protocol)
 {
@@ -610,7 +627,7 @@ lws_callback_on_writable_all_protocol_vhost(const struct lws_vhost *vhost,
 	return 0;
 }
 
-LWS_VISIBLE int
+int
 lws_callback_on_writable_all_protocol(const struct lws_context *context,
 				      const struct lws_protocols *protocol)
 {
