@@ -1,25 +1,28 @@
 /*
  * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2010-2019 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2019 Andy Green <andy@warmcat.com>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation:
- *  version 2.1 of the License.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA  02110-1301  USA
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
  */
 
-#include "core/private.h"
+#include "private-lib-core.h"
 
 static int
 sul_compare(const lws_dll2_t *d, const lws_dll2_t *i)
@@ -104,29 +107,37 @@ lws_sul_schedule(struct lws_context *context, int tsi,
 lws_usec_t
 __lws_sul_service_ripe(lws_dll2_owner_t *own, lws_usec_t usnow)
 {
+	struct lws_context_per_thread *pt = (struct lws_context_per_thread *)
+			lws_container_of(own, struct lws_context_per_thread,
+					 pt_sul_owner);
+
+	if (pt->attach_owner.count)
+		lws_system_do_attach(pt);
+
 	while (lws_dll2_get_head(own)) {
-                /* .list is always first member in lws_sorted_usec_list_t */
-                lws_sorted_usec_list_t *sul = (lws_sorted_usec_list_t *)
-                                                        lws_dll2_get_head(own);
+
+		/* .list is always first member in lws_sorted_usec_list_t */
+		lws_sorted_usec_list_t *sul = (lws_sorted_usec_list_t *)
+							lws_dll2_get_head(own);
+
 		assert(sul->us); /* shouldn't be on the list otherwise */
+
 		if (sul->us > usnow)
-			/*
-			 * No need to look further if we met one later than now:
-			 * the list is sorted in ascending time order
-			 */
 			return sul->us - usnow;
 
 		/* his moment has come... remove him from timeout list */
-
 		lws_dll2_remove(&sul->list);
 		sul->us = 0;
+		pt->inside_lws_service = 1;
 		sul->cb(sul);
-                /*
-                 * The callback may have done any mixture of delete
-                 * and add sul entries... eg, close a wsi may pull out
-                 * multiple entries making iterating it statefully
-                 * unsafe.  Always restart at the current head of list.
-                 */
+		pt->inside_lws_service = 0;
+
+		/*
+		 * The callback may have done any mixture of delete
+		 * and add sul entries... eg, close a wsi may pull out
+		 * multiple entries making iterating it statefully
+		 * unsafe.  Always restart at the current head of list.
+		 */
 	}
 
 	/*
