@@ -1,25 +1,28 @@
 /*
  * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2010-2019 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2019 Andy Green <andy@warmcat.com>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation:
- *  version 2.1 of the License.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA  02110-1301  USA
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
  */
 
-#include "core/private.h"
+#include "private-lib-core.h"
 
 const struct lws_role_ops *available_roles[] = {
 #if defined(LWS_ROLE_H2)
@@ -36,6 +39,9 @@ const struct lws_role_ops *available_roles[] = {
 #endif
 #if defined(LWS_ROLE_RAW_PROXY)
 	&role_ops_raw_proxy,
+#endif
+#if defined(LWS_ROLE_MQTT) && defined(LWS_WITH_CLIENT)
+	&role_ops_mqtt,
 #endif
 	NULL
 };
@@ -65,6 +71,24 @@ const struct lws_protocols *available_abstract_protocols[] = {
 };
 #endif
 
+#if defined(LWS_WITH_SECURE_STREAMS)
+const struct lws_protocols *available_secstream_protocols[] = {
+#if defined(LWS_ROLE_H1)
+	&protocol_secstream_h1,
+#endif
+#if defined(LWS_ROLE_H2)
+	&protocol_secstream_h2,
+#endif
+#if defined(LWS_ROLE_WS)
+	&protocol_secstream_ws,
+#endif
+#if defined(LWS_ROLE_MQTT)
+	&protocol_secstream_mqtt,
+#endif
+	NULL
+};
+#endif
+
 static const char * const mount_protocols[] = {
 	"http://",
 	"https://",
@@ -86,8 +110,10 @@ lws_role_by_name(const char *name)
 	if (!strcmp(name, role_ops_raw_skt.name))
 		return &role_ops_raw_skt;
 
+#if defined(LWS_ROLE_RAW_FILE)
 	if (!strcmp(name, role_ops_raw_file.name))
 		return &role_ops_raw_file;
+#endif
 
 	return NULL;
 }
@@ -109,7 +135,6 @@ lws_role_call_alpn_negotiated(struct lws *wsi, const char *alpn)
 	return 0;
 }
 
-//#if !defined(LWS_WITHOUT_SERVER)
 int
 lws_role_call_adoption_bind(struct lws *wsi, int type, const char *prot)
 {
@@ -168,17 +193,19 @@ lws_role_call_adoption_bind(struct lws *wsi, int type, const char *prot)
 	    role_ops_raw_skt.adoption_bind(wsi, type, prot))
 		return 0;
 
+#if defined(LWS_ROLE_RAW_FILE)
+
 	/* fall back to raw file role if, eg, h1 not configured */
 
 	if (role_ops_raw_file.adoption_bind &&
 	    role_ops_raw_file.adoption_bind(wsi, type, prot))
 		return 0;
+#endif
 
 	return 1;
 }
-//#endif
 
-#if !defined(LWS_WITHOUT_CLIENT)
+#if defined(LWS_WITH_CLIENT)
 int
 lws_role_call_client_bind(struct lws *wsi,
 			  const struct lws_client_connect_info *i)
@@ -203,7 +230,7 @@ lws_role_call_client_bind(struct lws *wsi,
 }
 #endif
 
-LWS_VISIBLE void *
+void *
 lws_protocol_vh_priv_zalloc(struct lws_vhost *vhost,
 			    const struct lws_protocols *prot, int size)
 {
@@ -235,7 +262,7 @@ lws_protocol_vh_priv_zalloc(struct lws_vhost *vhost,
 	return vhost->protocol_vh_privs[n];
 }
 
-LWS_VISIBLE void *
+void *
 lws_protocol_vh_priv_get(struct lws_vhost *vhost,
 			 const struct lws_protocols *prot)
 {
@@ -279,107 +306,117 @@ lws_vhost_protocol_options(struct lws_vhost *vh, const char *name)
 	return NULL;
 }
 
+int
+lws_protocol_init_vhost(struct lws_vhost *vh, int *any)
+{
+	const struct lws_protocol_vhost_options *pvo, *pvo1;
+	struct lws *wsi = vh->context->pt[0].fake_wsi;
+	int n;
+
+	wsi->context = vh->context;
+	wsi->vhost = vh;
+
+	/* initialize supported protocols on this vhost */
+
+	for (n = 0; n < vh->count_protocols; n++) {
+		wsi->protocol = &vh->protocols[n];
+		if (!vh->protocols[n].name)
+			continue;
+		pvo = lws_vhost_protocol_options(vh, vh->protocols[n].name);
+		if (pvo) {
+			/*
+			 * linked list of options specific to
+			 * vh + protocol
+			 */
+			pvo1 = pvo;
+			pvo = pvo1->options;
+
+			while (pvo) {
+				lwsl_debug(
+					"    vhost \"%s\", "
+					"protocol \"%s\", "
+					"option \"%s\"\n",
+						vh->name,
+						vh->protocols[n].name,
+						pvo->name);
+
+				if (!strcmp(pvo->name, "default")) {
+					lwsl_info("Setting default "
+					   "protocol for vh %s to %s\n",
+					   vh->name,
+					   vh->protocols[n].name);
+					vh->default_protocol_index = n;
+				}
+				if (!strcmp(pvo->name, "raw")) {
+					lwsl_info("Setting raw "
+					   "protocol for vh %s to %s\n",
+					   vh->name,
+					   vh->protocols[n].name);
+					vh->raw_protocol_index = n;
+				}
+				pvo = pvo->next;
+			}
+
+			pvo = pvo1->options;
+		}
+
+#if defined(LWS_WITH_TLS)
+		if (any)
+			*any |= !!vh->tls.ssl_ctx;
+#endif
+
+		/*
+		 * inform all the protocols that they are doing their
+		 * one-time initialization if they want to.
+		 *
+		 * NOTE the wsi is all zeros except for the context, vh
+		 * + protocol ptrs so lws_get_context(wsi) etc can work
+		 */
+		if (vh->protocols[n].callback(wsi,
+				LWS_CALLBACK_PROTOCOL_INIT, NULL,
+				(void *)pvo, 0)) {
+			if (vh->protocol_vh_privs[n]) {
+				lws_free(vh->protocol_vh_privs[n]);
+				vh->protocol_vh_privs[n] = NULL;
+			}
+			lwsl_err("%s: protocol %s failed init\n",
+				 __func__, vh->protocols[n].name);
+
+			return 1;
+		}
+	}
+
+	vh->created_vhost_protocols = 1;
+
+	return 0;
+}
+
 /*
  * inform every vhost that hasn't already done it, that
  * his protocols are initializing
  */
-LWS_VISIBLE int
+int
 lws_protocol_init(struct lws_context *context)
 {
 	struct lws_vhost *vh = context->vhost_list;
-	const struct lws_protocol_vhost_options *pvo, *pvo1;
-	struct lws wsi;
-	int n, any = 0;
+	int any = 0;
 
 	if (context->doing_protocol_init)
 		return 0;
 
 	context->doing_protocol_init = 1;
 
-	memset(&wsi, 0, sizeof(wsi));
-	wsi.context = context;
-
 	lwsl_info("%s\n", __func__);
 
 	while (vh) {
-		wsi.vhost = vh;
 
 		/* only do the protocol init once for a given vhost */
 		if (vh->created_vhost_protocols ||
-		    (vh->options & LWS_SERVER_OPTION_SKIP_PROTOCOL_INIT))
+		    (lws_check_opt(vh->options, LWS_SERVER_OPTION_SKIP_PROTOCOL_INIT)))
 			goto next;
 
-		/* initialize supported protocols on this vhost */
-
-		for (n = 0; n < vh->count_protocols; n++) {
-			wsi.protocol = &vh->protocols[n];
-			if (!vh->protocols[n].name)
-				continue;
-			pvo = lws_vhost_protocol_options(vh,
-							 vh->protocols[n].name);
-			if (pvo) {
-				/*
-				 * linked list of options specific to
-				 * vh + protocol
-				 */
-				pvo1 = pvo;
-				pvo = pvo1->options;
-
-				while (pvo) {
-					lwsl_debug(
-						"    vhost \"%s\", "
-						"protocol \"%s\", "
-						"option \"%s\"\n",
-							vh->name,
-							vh->protocols[n].name,
-							pvo->name);
-
-					if (!strcmp(pvo->name, "default")) {
-						lwsl_info("Setting default "
-						   "protocol for vh %s to %s\n",
-						   vh->name,
-						   vh->protocols[n].name);
-						vh->default_protocol_index = n;
-					}
-					if (!strcmp(pvo->name, "raw")) {
-						lwsl_info("Setting raw "
-						   "protocol for vh %s to %s\n",
-						   vh->name,
-						   vh->protocols[n].name);
-						vh->raw_protocol_index = n;
-					}
-					pvo = pvo->next;
-				}
-
-				pvo = pvo1->options;
-			}
-
-#if defined(LWS_WITH_TLS)
-			any |= !!vh->tls.ssl_ctx;
-#endif
-
-			/*
-			 * inform all the protocols that they are doing their
-			 * one-time initialization if they want to.
-			 *
-			 * NOTE the wsi is all zeros except for the context, vh
-			 * + protocol ptrs so lws_get_context(wsi) etc can work
-			 */
-			if (vh->protocols[n].callback(&wsi,
-					LWS_CALLBACK_PROTOCOL_INIT, NULL,
-					(void *)pvo, 0)) {
-				if (vh->protocol_vh_privs[n]) {
-					lws_free(vh->protocol_vh_privs[n]);
-					vh->protocol_vh_privs[n] = NULL;
-				}
-				lwsl_err("%s: protocol %s failed init\n",
-					 __func__, vh->protocols[n].name);
-
-				return 1;
-			}
-		}
-
-		vh->created_vhost_protocols = 1;
+		if (lws_protocol_init_vhost(vh, &any))
+			return 1;
 next:
 		vh = vh->vhost_next;
 	}
@@ -391,8 +428,10 @@ next:
 
 	context->protocol_init_done = 1;
 
+#if defined(LWS_WITH_SERVER)
 	if (any)
 		lws_tls_check_all_cert_lifetimes(context);
+#endif
 
 	return 0;
 }
@@ -423,7 +462,7 @@ static const struct lws_protocols protocols_dummy[] = {
 #undef LWS_HAVE_GETENV
 #endif
 
-LWS_VISIBLE struct lws_vhost *
+struct lws_vhost *
 lws_create_vhost(struct lws_context *context,
 		 const struct lws_context_creation_info *info)
 {
@@ -435,10 +474,14 @@ lws_create_vhost(struct lws_context *context,
 	struct lws_plugin *plugin = context->plugin_list;
 #endif
 	struct lws_protocols *lwsp;
-	int m, f = !info->pvo, fx = 0, abs_pcol_count = 0;
+	int m, f = !info->pvo, fx = 0, abs_pcol_count = 0, sec_pcol_count = 0;
 	char buf[96];
-#if !defined(LWS_WITHOUT_CLIENT) && defined(LWS_HAVE_GETENV)
+#if defined(LWS_CLIENT_HTTP_PROXYING) && defined(LWS_WITH_CLIENT) \
+	&& defined(LWS_HAVE_GETENV)
 	char *p;
+#endif
+#if defined(LWS_WITH_SYS_ASYNC_DNS)
+	extern struct lws_protocols lws_async_dns_protocol;
 #endif
 	int n;
 
@@ -462,14 +505,19 @@ lws_create_vhost(struct lws_context *context,
 	vh->http.error_document_404 = info->error_document_404;
 #endif
 
-	if (info->options & LWS_SERVER_OPTION_ONLY_RAW)
+	if (lws_check_opt(info->options, LWS_SERVER_OPTION_ONLY_RAW))
 		lwsl_info("%s set to only support RAW\n", vh->name);
 
 	vh->iface = info->iface;
-#if !defined(LWS_WITH_ESP32) && \
-    !defined(OPTEE_TA) && !defined(WIN32)
+#if !defined(LWS_PLAT_FREERTOS) && !defined(OPTEE_TA) && !defined(WIN32)
 	vh->bind_iface = info->bind_iface;
 #endif
+	/* apply the context default lws_retry */
+
+	if (info->retry_and_idle_policy)
+		vh->retry_policy = info->retry_and_idle_policy;
+	else
+		vh->retry_policy = &context->default_retry;
 
 	/*
 	 * let's figure out how many protocols the user is handing us, using the
@@ -551,17 +599,21 @@ lws_create_vhost(struct lws_context *context,
 #if defined(LWS_WITH_ABSTRACT)
 	abs_pcol_count = (int)LWS_ARRAY_SIZE(available_abstract_protocols) - 1;
 #endif
+#if defined(LWS_WITH_SECURE_STREAMS)
+	sec_pcol_count = (int)LWS_ARRAY_SIZE(available_secstream_protocols) - 1;
+#endif
 
 	/*
 	 * give the vhost a unified list of protocols including:
 	 *
+	 * - internal, async_dns if enabled (first vhost only)
 	 * - internal, abstracted ones
 	 * - the ones that came from plugins
 	 * - his user protocols
 	 */
 	lwsp = lws_zalloc(sizeof(struct lws_protocols) *
 				(vh->count_protocols +
-				   abs_pcol_count +
+				   abs_pcol_count + sec_pcol_count +
 				   context->plugin_protocol_count +
 				   fx + 1),
 			  "vhost-specific plugin table");
@@ -591,6 +643,24 @@ lws_create_vhost(struct lws_context *context,
 		vh->count_protocols++;
 	}
 #endif
+	/*
+	 * 3: async dns protocol (first vhost only)
+	 */
+#if defined(LWS_WITH_SYS_ASYNC_DNS)
+	if (!context->vhost_list) {
+		memcpy(&lwsp[m++], &lws_async_dns_protocol,
+		       sizeof(struct lws_protocols));
+		vh->count_protocols++;
+	}
+#endif
+
+#if defined(LWS_WITH_SECURE_STREAMS)
+	for (n = 0; n < sec_pcol_count; n++) {
+		memcpy(&lwsp[m++], available_secstream_protocols[n],
+		       sizeof(*lwsp));
+		vh->count_protocols++;
+	}
+#endif
 
 	/*
 	 * 3: For compatibility, all protocols enabled on vhost if only
@@ -671,18 +741,17 @@ lws_create_vhost(struct lws_context *context,
 	}
 
 	vh->listen_port = info->port;
-#if defined(LWS_ROLE_H1) || defined(LWS_ROLE_H2)
-	vh->http.http_proxy_port = 0;
-	vh->http.http_proxy_address[0] = '\0';
-#endif
+
 #if defined(LWS_WITH_SOCKS5)
 	vh->socks_proxy_port = 0;
 	vh->socks_proxy_address[0] = '\0';
 #endif
 
-#if !defined(LWS_WITHOUT_CLIENT)
+#if defined(LWS_WITH_CLIENT) && defined(LWS_CLIENT_HTTP_PROXYING)
 	/* either use proxy from info, or try get it from env var */
 #if defined(LWS_ROLE_H1) || defined(LWS_ROLE_H2)
+	vh->http.http_proxy_port = 0;
+	vh->http.http_proxy_address[0] = '\0';
 	/* http proxy */
 	if (info->http_proxy_address) {
 		/* override for backwards compatibility */
@@ -693,29 +762,20 @@ lws_create_vhost(struct lws_context *context,
 #endif
 	{
 #ifdef LWS_HAVE_GETENV
-		p = getenv("http_proxy");
+#if defined(__COVERITY__)
+		p = NULL;
+#else
+		p = getenv("http_proxy"); /* coverity[tainted_scalar] */
 		if (p) {
 			lws_strncpy(buf, p, sizeof(buf));
-
 			lws_set_proxy(vh, buf);
 		}
+#endif
 #endif
 	}
 #endif
 #if defined(LWS_WITH_SOCKS5)
-	/* socks proxy */
-	if (info->socks_proxy_address) {
-		/* override for backwards compatibility */
-		if (info->socks_proxy_port)
-			vh->socks_proxy_port = info->socks_proxy_port;
-		lws_set_socks(vh, info->socks_proxy_address);
-	} else {
-#ifdef LWS_HAVE_GETENV
-		p = getenv("socks_proxy");
-		if (p && strlen(p) > 0 && strlen(p) < 95)
-			lws_set_socks(vh, p);
-#endif
-	}
+	lws_socks5c_ads_server(vh, info);
 #endif
 
 	vh->ka_time = info->ka_time;
@@ -752,6 +812,7 @@ lws_create_vhost(struct lws_context *context,
 		lwsl_err("%s: lws_context_init_client_ssl failed\n", __func__);
 		goto bail1;
 	}
+#if defined(LWS_WITH_SERVER)
 	lws_context_lock(context, "create_vhost");
 	n = _lws_vhost_init_server(info, vh);
 	lws_context_unlock(context);
@@ -759,6 +820,8 @@ lws_create_vhost(struct lws_context *context,
 		lwsl_err("init server failed\n");
 		goto bail1;
 	}
+#endif
+	n = !!context->vhost_list;
 
 	while (1) {
 		if (!(*vh1)) {
@@ -768,6 +831,11 @@ lws_create_vhost(struct lws_context *context,
 		vh1 = &(*vh1)->vhost_next;
 	};
 
+#if defined(LWS_WITH_SYS_ASYNC_DNS)
+	if (!n && lws_async_dns_init(context))
+		goto bail1;
+#endif
+
 	/* for the case we are adding a vhost much later, after server init */
 
 	if (context->protocol_init_done)
@@ -791,7 +859,7 @@ bail:
 	return NULL;
 }
 
-LWS_VISIBLE int
+int
 lws_init_vhost_client_ssl(const struct lws_context_creation_info *info,
 			  struct lws_vhost *vhost)
 {
@@ -803,13 +871,13 @@ lws_init_vhost_client_ssl(const struct lws_context_creation_info *info,
 	return lws_context_init_client_ssl(&i, vhost);
 }
 
-LWS_VISIBLE void
+void
 lws_cancel_service_pt(struct lws *wsi)
 {
 	lws_plat_pipe_signal(wsi);
 }
 
-LWS_VISIBLE void
+void
 lws_cancel_service(struct lws_context *context)
 {
 	struct lws_context_per_thread *pt = &context->pt[0];
@@ -838,9 +906,14 @@ lws_create_event_pipes(struct lws_context *context)
 	 * not bound to a vhost or protocol (both are NULL)
 	 */
 
+#if LWS_MAX_SMP > 1
 	for (n = 0; n < context->count_threads; n++) {
+#else
+	n = 0;
+	{
+#endif
 		if (context->pt[n].pipe_wsi)
-			continue;
+			return 0;
 
 		wsi = lws_zalloc(sizeof(*wsi), "event pipe wsi");
 		if (!wsi) {
@@ -857,7 +930,7 @@ lws_create_event_pipes(struct lws_context *context)
 		context->pt[n].pipe_wsi = wsi;
 		context->count_wsi_allocated++;
 
-		if (lws_plat_pipe_create(wsi))
+		if (!lws_plat_pipe_create(wsi)) {
 			/*
 			 * platform code returns 0 if it actually created pipes
 			 * and initialized pt->dummy_pipe_fds[].  If it used
@@ -866,19 +939,17 @@ lws_create_event_pipes(struct lws_context *context)
 			 * related to dummy_pipe_fds[], adding it to the fds,
 			 * etc.
 			 */
-			continue;
 
-		wsi->desc.sockfd = context->pt[n].dummy_pipe_fds[0];
-		lwsl_debug("event pipe fd %d\n", wsi->desc.sockfd);
+			wsi->desc.sockfd = context->pt[n].dummy_pipe_fds[0];
+			lwsl_debug("event pipe fd %d\n", wsi->desc.sockfd);
 
-#if !defined(LWS_AMAZON_RTOS)
-		if (context->event_loop_ops->accept)
-			if (context->event_loop_ops->accept(wsi))
+			if (context->event_loop_ops->sock_accept)
+				if (context->event_loop_ops->sock_accept(wsi))
+					return 1;
+
+			if (__insert_wsi_socket_into_fds(context, wsi))
 				return 1;
-#endif
-
-		if (__insert_wsi_socket_into_fds(context, wsi))
-			return 1;
+		}
 	}
 
 	return 0;
@@ -888,9 +959,12 @@ void
 lws_destroy_event_pipe(struct lws *wsi)
 {
 	lwsl_info("%s\n", __func__);
-	__remove_wsi_socket_from_fds(wsi);
 
-	if (wsi->context->event_loop_ops->wsi_logical_close) {
+	if (lws_socket_is_valid(wsi->desc.sockfd))
+		__remove_wsi_socket_from_fds(wsi);
+
+	if (!wsi->context->event_loop_ops->destroy_wsi &&
+	    wsi->context->event_loop_ops->wsi_logical_close) {
 		wsi->context->event_loop_ops->wsi_logical_close(wsi);
 		lws_plat_pipe_close(wsi);
 		return;
@@ -918,7 +992,6 @@ lws_vhost_destroy1(struct lws_vhost *vh)
 
 	lws_vhost_lock(vh); /* -------------- vh { */
 
-	vh->being_destroyed = 1;
 #if defined(LWS_WITH_NETWORK)
 	/*
 	 * PHASE 1: take down or reassign any listen wsi
@@ -1000,6 +1073,24 @@ __lws_vhost_destroy2(struct lws_vhost *vh)
 	struct lws wsi;
 	int n;
 
+	vh->being_destroyed = 0;
+
+#if defined(LWS_WITH_CLIENT)
+	/*
+	 * destroy any wsi that are associated with us but have no socket
+	 * (and will otherwise be missed for destruction)
+	 */
+	lws_start_foreach_dll_safe(struct lws_dll2 *, d, d1,
+			      vh->vh_awaiting_socket_owner.head) {
+		struct lws *w =
+			lws_container_of(d, struct lws, vh_awaiting_socket);
+
+		lws_close_free_wsi(w, LWS_CLOSE_STATUS_NOSTATUS,
+				   "awaiting skt");
+
+	} lws_end_foreach_dll_safe(d, d1);
+#endif
+
 	/*
 	 * destroy any pending timed events
 	 */
@@ -1090,7 +1181,10 @@ __lws_vhost_destroy2(struct lws_vhost *vh)
 	lws_ssl_SSL_CTX_destroy(vh);
 	lws_free(vh->same_vh_protocol_owner);
 
-	if (context->plugin_list ||
+	if (
+#if defined(LWS_WITH_PLUGINS)
+		context->plugin_list ||
+#endif
 	    (context->options & LWS_SERVER_OPTION_EXPLICIT_VHOSTS) ||
 	    vh->allocated_vhost_protocols)
 		lws_free((void *)vh->protocols);
@@ -1209,7 +1303,7 @@ lws_check_deferred_free(struct lws_context *context, int tsi, int force)
 }
 
 
-LWS_VISIBLE void
+void
 lws_vhost_destroy(struct lws_vhost *vh)
 {
 	struct lws_deferred_free *df = lws_malloc(sizeof(*df), "deferred free");
@@ -1222,6 +1316,8 @@ lws_vhost_destroy(struct lws_vhost *vh)
 
 	lws_vhost_destroy1(vh);
 
+	lwsl_debug("%s: count_bound_wsi %d\n", __func__, vh->count_bound_wsi);
+
 	if (!vh->count_bound_wsi) {
 		/*
 		 * After listen handoff, there are already no wsi bound to this
@@ -1248,20 +1344,20 @@ out:
 }
 
 
-LWS_EXTERN void *
+void *
 lws_vhost_user(struct lws_vhost *vhost)
 {
 	return vhost->user;
 }
 
-LWS_VISIBLE LWS_EXTERN int
+int
 lws_get_vhost_listen_port(struct lws_vhost *vhost)
 {
 	return vhost->listen_port;
 }
 
-
-LWS_VISIBLE LWS_EXTERN void
+#if defined(LWS_WITH_SERVER)
+void
 lws_context_deprecate(struct lws_context *context, lws_reload_func cb)
 {
 	struct lws_vhost *vh = context->vhost_list, *vh1;
@@ -1301,8 +1397,10 @@ lws_context_deprecate(struct lws_context *context, lws_reload_func cb)
 	context->deprecated = 1;
 	context->deprecation_cb = cb;
 }
+#endif
 
 #if defined(LWS_WITH_NETWORK)
+
 struct lws_vhost *
 lws_get_vhost_by_name(struct lws_context *context, const char *name)
 {
@@ -1315,4 +1413,171 @@ lws_get_vhost_by_name(struct lws_context *context, const char *name)
 
 	return NULL;
 }
+
+
+#if defined(LWS_WITH_CLIENT)
+/*
+ * This is the logic checking to see if the new connection wsi should have a
+ * pipelining or muxing relationship with an existing "active connection" to
+ * the same endpoint under the same conditions.
+ *
+ * This was originally in the client code but since the list is held on the
+ * vhost (to ensure the same client tls ctx is involved) it's cleaner in vhost.c
+ */
+
+int
+lws_vhost_active_conns(struct lws *wsi, struct lws **nwsi, const char *adsin)
+{
+	if (!lws_dll2_is_detached(&wsi->dll2_cli_txn_queue)) {
+		struct lws *w = lws_container_of(
+				wsi->dll2_cli_txn_queue.owner, struct lws,
+				dll2_cli_txn_queue_owner);
+		*nwsi = w;
+
+		return ACTIVE_CONNS_QUEUED;
+	}
+
+#if defined(LWS_ROLE_H2) || defined(LWS_ROLE_MQTT)
+	if (wsi->mux.parent_wsi) {
+		/*
+		 * We already decided...
+		 */
+
+		*nwsi = wsi->mux.parent_wsi;
+
+		return ACTIVE_CONNS_MUXED;
+	}
+#endif
+
+	lws_vhost_lock(wsi->vhost); /* ----------------------------------- { */
+
+	lws_start_foreach_dll_safe(struct lws_dll2 *, d, d1,
+				   wsi->vhost->dll_cli_active_conns_owner.head) {
+		struct lws *w = lws_container_of(d, struct lws,
+						 dll_cli_active_conns);
+
+		lwsl_debug("%s: check %p %p %s %s %d %d\n", __func__, wsi, w,
+			    adsin, w->cli_hostname_copy, wsi->c_port, w->c_port);
+
+		if (w != wsi &&
+		    /*
+		     * "same internet protocol"... this is a bit tricky,
+		     * since h2 start out as h1
+		     */
+		    (w->role_ops == wsi->role_ops ||
+		     (lwsi_role_http(w) && lwsi_role_http(wsi))) &&
+		    w->cli_hostname_copy &&
+		    !strcmp(adsin, w->cli_hostname_copy) &&
+#if defined(LWS_WITH_TLS)
+		    (wsi->tls.use_ssl & LCCSCF_USE_SSL) ==
+		     (w->tls.use_ssl & LCCSCF_USE_SSL) &&
+#endif
+		    wsi->c_port == w->c_port) {
+
+			/*
+			 * There's already an active connection.
+			 *
+			 * The server may have told the existing active
+			 * connection that it doesn't support pipelining...
+			 */
+			if (w->keepalive_rejected) {
+				lwsl_notice("defeating pipelining due to no "
+					  "keepalive on server\n");
+				goto solo;
+			}
+
+#if defined(LWS_WITH_HTTP2)
+			/*
+			 * h2: if in usable state already: just use it without
+			 *     going through the queue
+			 */
+			if (w->client_h2_alpn && w->client_mux_migrated &&
+			    (lwsi_state(w) == LRS_H2_WAITING_TO_SEND_HEADERS ||
+			     lwsi_state(w) == LRS_ESTABLISHED ||
+			     lwsi_state(w) == LRS_IDLING)) {
+
+				lwsl_notice("%s: just join h2 directly 0x%x\n",
+						__func__, lwsi_state(w));
+
+				if (lwsi_state(w) == LRS_IDLING) {
+					// lwsi_set_state(w, LRS_ESTABLISHED);
+					_lws_generic_transaction_completed_active_conn(&w);
+				}
+
+				//lwsi_set_state(w, LRS_H1C_ISSUE_HANDSHAKE2);
+
+				wsi->client_h2_alpn = 1;
+				lws_wsi_h2_adopt(w, wsi);
+				lws_vhost_unlock(wsi->vhost); /* } ---------- */
+
+				*nwsi = w;
+
+				return ACTIVE_CONNS_MUXED;
+			}
+#endif
+
+#if defined(LWS_ROLE_MQTT)
+			/*
+			 * MQTT: if in usable state already: just use it without
+			 *	 going through the queue
+			 */
+
+			if (lwsi_role_mqtt(wsi) && w->client_mux_migrated &&
+			    lwsi_state(w) == LRS_ESTABLISHED) {
+
+				if (lws_wsi_mqtt_adopt(w, wsi)) {
+					lwsl_notice("%s: join mqtt directly\n", __func__);
+					lws_dll2_remove(&wsi->dll2_cli_txn_queue);
+					wsi->client_mux_substream = 1;
+
+					lws_vhost_unlock(wsi->vhost); /* } ---------- */
+
+
+					return ACTIVE_CONNS_MUXED;
+				}
+			}
+#endif
+
+			/*
+			 * If the connection is viable but not yet in a usable
+			 * state, let's attach ourselves to it and wait for it
+			 * to get there or fail.
+			 */
+
+			lwsl_notice("%s: apply %p to txn queue on %p state 0x%lx\n",
+				  __func__, wsi, w, (unsigned long)w->wsistate);
+			/*
+			 * ...let's add ourselves to his transaction queue...
+			 * we are adding ourselves at the TAIL
+			 */
+			lws_dll2_add_tail(&wsi->dll2_cli_txn_queue,
+					  &w->dll2_cli_txn_queue_owner);
+
+			if (lwsi_state(w) == LRS_IDLING) {
+				// lwsi_set_state(w, LRS_ESTABLISHED);
+				_lws_generic_transaction_completed_active_conn(&w);
+			}
+
+			/*
+			 * For eg, h1 next we'd pipeline our headers out on him,
+			 * and wait for our turn at client transaction_complete
+			 * to take over parsing the rx.
+			 */
+			lws_vhost_unlock(wsi->vhost); /* } ---------- */
+
+			*nwsi = w;
+
+			return ACTIVE_CONNS_QUEUED;
+		}
+
+	} lws_end_foreach_dll_safe(d, d1);
+
+solo:
+	lws_vhost_unlock(wsi->vhost); /* } ---------------------------------- */
+
+	/* there is nobody already connected in the same way */
+
+	return ACTIVE_CONNS_SOLO;
+}
+#endif
 #endif
