@@ -1,33 +1,40 @@
 /*
  * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2010-2019 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2020 Andy Green <andy@warmcat.com>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation:
- *  version 2.1 of the License.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA  02110-1301  USA
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
  */
 
-#include "core/private.h"
+#include "private-lib-core.h"
 
 #ifndef LWS_BUILD_HASH
 #define LWS_BUILD_HASH "unknown-build-hash"
 #endif
 
-
 static const char *library_version = LWS_LIBRARY_VERSION " " LWS_BUILD_HASH;
 
+#if defined(LWS_WITH_NETWORK)
+/* in ms */
+static uint32_t default_backoff_table[] = { 1000, 3000, 9000, 17000 };
+#endif
+
 /**
  * lws_get_library_version: get version and git hash library built from
  *
@@ -35,7 +42,7 @@ static const char *library_version = LWS_LIBRARY_VERSION " " LWS_BUILD_HASH;
  *	representing the library version followed by the git head hash it
  *	was built from
  */
-LWS_VISIBLE const char *
+const char *
 lws_get_library_version(void)
 {
 	return library_version;
@@ -66,21 +73,155 @@ lws_sul_peer_limits_cb(lws_sorted_usec_list_t *sul)
 }
 #endif
 
+#if defined(LWS_WITH_NETWORK)
 
-LWS_VISIBLE struct lws_context *
+#if defined(_DEBUG)
+static const char * system_state_names[] = {
+	"undef",
+	"CONTEXT_CREATED",
+	"INITIALIZED",
+	"IFACE_COLDPLUG",
+	"DHCP",
+	"TIME_VALID",
+	"POLICY_VALID",
+	"REGISTERED",
+	"AUTH1",
+	"AUTH2",
+	"OPERATIONAL",
+	"POLICY_INVALID"
+};
+#endif
+
+/*
+ * Handle provoking protocol init when we pass through the right system state
+ */
+
+static int
+lws_state_notify_protocol_init(struct lws_state_manager *mgr,
+			       struct lws_state_notify_link *link, int current,
+			       int target)
+{
+	struct lws_context *context = lws_container_of(mgr, struct lws_context,
+						       mgr_system);
+	int n;
+
+	/*
+	 * Deal with any attachments that were waiting for the right state
+	 * to come along
+	 */
+
+	for (n = 0; n < context->count_threads; n++)
+		lws_system_do_attach(&context->pt[n]);
+
+#if defined(LWS_WITH_SYS_DHCP_CLIENT)
+	if (target == LWS_SYSTATE_DHCP) {
+		/*
+		 * Don't let it past here until at least one iface has been
+		 * configured for operation with DHCP
+		 */
+
+		if (!lws_dhcpc_status(context, NULL))
+			return 1;
+	}
+#endif
+
+#if defined(LWS_WITH_SECURE_STREAMS_SYS_AUTH_API_AMAZON_COM)
+	/*
+	 * Skip this if we are running something without the policy for it
+	 *
+	 * If root token is empty, skip too.
+	 */
+	if (target == LWS_SYSTATE_AUTH1 &&
+	    context->pss_policies &&
+	    !lws_system_blob_get_size(lws_system_get_blob(context,
+						          LWS_SYSBLOB_TYPE_AUTH,
+						          0)) &&
+	    lws_system_blob_get_size(lws_system_get_blob(context,
+						          LWS_SYSBLOB_TYPE_AUTH,
+						          1))) {
+		lwsl_info("%s: AUTH1 state triggering api.amazon.com auth\n", __func__);
+		/*
+		 * Start trying to acquire it if it's not already in progress
+		 * returns nonzero if we determine it's not needed
+		 */
+		if (!lws_ss_sys_auth_api_amazon_com(context))
+			return 1;
+	}
+#endif
+
+#if defined(LWS_WITH_SECURE_STREAMS)
+	/*
+	 * Skip this if we are running something without the policy for it
+	 */
+	if (target == LWS_SYSTATE_POLICY_VALID &&
+	    context->pss_policies && !context->policy_updated) {
+		/*
+		 * Start trying to acquire it if it's not already in progress
+		 * returns nonzero if we determine it's not needed
+		 */
+		if (!lws_ss_sys_fetch_policy(context))
+			return 1;
+	}
+#endif
+
+	/* protocol part */
+
+	if (context->protocol_init_done)
+		return 0;
+
+	if (target != LWS_SYSTATE_POLICY_VALID)
+		return 0;
+
+	lwsl_info("%s: doing protocol init on POLICY_VALID\n", __func__);
+	lws_protocol_init(context);
+
+	return 0;
+}
+
+static void
+lws_context_creation_completion_cb(lws_sorted_usec_list_t *sul)
+{
+	struct lws_context *context = lws_container_of(sul, struct lws_context,
+						       sul_system_state);
+
+	/* if nothing is there to intercept anything, go all the way */
+	lws_state_transition_steps(&context->mgr_system,
+				   LWS_SYSTATE_OPERATIONAL);
+}
+#endif
+
+struct lws_context *
 lws_create_context(const struct lws_context_creation_info *info)
 {
 	struct lws_context *context = NULL;
+#if defined(LWS_WITH_FILE_OPS)
 	struct lws_plat_file_ops *prev;
+#endif
 #ifndef LWS_NO_DAEMONIZE
 	pid_t pid_daemon = get_daemonize_pid();
 #endif
 #if defined(LWS_WITH_NETWORK)
-	int n;
+	int n, count_threads = 1;
+	uint8_t *u;
 #endif
 #if defined(__ANDROID__)
 	struct rlimit rt;
 #endif
+	size_t s1 = 4096, size = sizeof(struct lws_context);
+	int lpf = info->fd_limit_per_thread;
+
+	if (lpf) {
+		lpf+= 2;
+#if defined(LWS_WITH_SYS_ASYNC_DNS)
+		lpf++;
+#endif
+#if defined(LWS_WITH_SYS_NTPCLIENT)
+		lpf++;
+#endif
+#if defined(LWS_WITH_SYS_DHCP_CLIENT)
+		lpf++;
+#endif
+	}
 
 	lwsl_info("Initial logging level %d\n", log_level);
 	lwsl_info("Libwebsockets version: %s\n", library_version);
@@ -95,7 +236,6 @@ lws_create_context(const struct lws_context_creation_info *info)
 #endif
 
 	lwsl_info(" LWS_DEF_HEADER_LEN    : %u\n", LWS_DEF_HEADER_LEN);
-	lwsl_info(" LWS_MAX_PROTOCOLS     : %u\n", LWS_MAX_PROTOCOLS);
 	lwsl_info(" LWS_MAX_SMP           : %u\n", LWS_MAX_SMP);
 	lwsl_info(" sizeof (*info)        : %ld\n", (long)sizeof(*info));
 #if defined(LWS_WITH_STATS)
@@ -110,7 +250,21 @@ lws_create_context(const struct lws_context_creation_info *info)
 	if (lws_plat_context_early_init())
 		return NULL;
 
-	context = lws_zalloc(sizeof(struct lws_context), "context");
+#if defined(LWS_WITH_NETWORK)
+	if (info->count_threads)
+		count_threads = info->count_threads;
+
+	if (count_threads > LWS_MAX_SMP)
+		count_threads = LWS_MAX_SMP;
+
+	if (info->pt_serv_buf_size)
+		s1 = info->pt_serv_buf_size;
+
+	/* pt fakewsi and the pt serv buf allocations ride after the context */
+	size += count_threads * (s1 + sizeof(struct lws));
+#endif
+
+	context = lws_zalloc(size, "context");
 	if (!context) {
 		lwsl_err("No memory for websocket context\n");
 		return NULL;
@@ -121,12 +275,47 @@ lws_create_context(const struct lws_context_creation_info *info)
 	context->username = info->username;
 	context->groupname = info->groupname;
 	context->system_ops = info->system_ops;
+	context->pt_serv_buf_size = (unsigned int)s1;
+	context->udp_loss_sim_tx_pc = info->udp_loss_sim_tx_pc;
+	context->udp_loss_sim_rx_pc = info->udp_loss_sim_rx_pc;
+
+	if (context->udp_loss_sim_tx_pc || context->udp_loss_sim_rx_pc)
+		lwsl_warn("%s: simulating udp loss tx: %d%%, rx: %d%%\n",
+			  __func__, context->udp_loss_sim_tx_pc,
+			  context->udp_loss_sim_rx_pc);
+
+#if defined(LWS_WITH_SECURE_STREAMS_PROXY_API)
+	context->ss_proxy_bind = info->ss_proxy_bind;
+	context->ss_proxy_port = info->ss_proxy_port;
+	context->ss_proxy_address = info->ss_proxy_address;
+	lwsl_notice("%s: using ss proxy bind '%s', port %d, ads '%s'\n",
+			__func__, context->ss_proxy_bind, context->ss_proxy_port,
+			context->ss_proxy_address);
+#endif
+
+#if defined(LWS_WITH_NETWORK)
+	context->count_threads = count_threads;
+#if defined(LWS_WITH_DETAILED_LATENCY)
+	context->detailed_latency_cb = info->detailed_latency_cb;
+	context->detailed_latency_filepath = info->detailed_latency_filepath;
+	context->latencies_fd = -1;
+#endif
+#if defined(LWS_WITHOUT_EXTENSIONS)
+        if (info->extensions)
+                lwsl_warn("%s: LWS_WITHOUT_EXTENSIONS but extensions ptr set\n", __func__);
+#endif
+#endif
+
+#if defined(LWS_WITH_SECURE_STREAMS)
+	context->pss_policies_json = info->pss_policies_json;
+	context->pss_plugins = info->pss_plugins;
+#endif
 
 	/* if he gave us names, set the uid / gid */
 	if (lws_plat_drop_app_privileges(context, 0))
 		goto bail;
 
-lwsl_info("context created\n");
+	lwsl_info("context created\n");
 #if defined(LWS_WITH_TLS) && defined(LWS_WITH_NETWORK)
 #if defined(LWS_WITH_MBEDTLS)
 	context->tls_ops = &tls_ops_mbedtls;
@@ -135,23 +324,19 @@ lwsl_info("context created\n");
 #endif
 #endif
 
-	if (info->pt_serv_buf_size)
-		context->pt_serv_buf_size = info->pt_serv_buf_size;
-	else
-		context->pt_serv_buf_size = 4096;
-
-#if defined(LWS_ROLE_H2)
-	role_ops_h2.init_context(context, info);
-#endif
-
 #if LWS_MAX_SMP > 1
 	lws_mutex_refcount_init(&context->mr);
 #endif
 
-#if defined(LWS_WITH_ESP32)
+#if defined(LWS_PLAT_FREERTOS)
+#if defined(LWS_AMAZON_RTOS)
+	context->last_free_heap = xPortGetFreeHeapSize();
+#else
 	context->last_free_heap = esp_get_free_heap_size();
 #endif
+#endif
 
+#if defined(LWS_WITH_FILE_OPS)
 	/* default to just the platform fops implementation */
 
 	context->fops_platform.LWS_FOP_OPEN	= _lws_plat_file_open;
@@ -182,8 +367,11 @@ lwsl_info("context created\n");
 	/* if user provided fops, tack them on the end of the list */
 	if (info->fops)
 		prev->next = info->fops;
+#endif
 
+#if defined(LWS_WITH_SERVER)
 	context->reject_service_keywords = info->reject_service_keywords;
+#endif
 	if (info->external_baggage_free_on_destroy)
 		context->external_baggage_free_on_destroy =
 			info->external_baggage_free_on_destroy;
@@ -204,43 +392,34 @@ lwsl_info("context created\n");
 	}
 #endif
 #if defined(__ANDROID__)
-		n = getrlimit(RLIMIT_NOFILE, &rt);
-		if (n == -1) {
-			lwsl_err("Get RLIMIT_NOFILE failed!\n");
+	n = getrlimit(RLIMIT_NOFILE, &rt);
+	if (n == -1) {
+		lwsl_err("Get RLIMIT_NOFILE failed!\n");
 
-			return NULL;
-		}
-		context->max_fds = rt.rlim_cur;
+		return NULL;
+	}
+	context->max_fds = rt.rlim_cur;
 #else
 #if defined(WIN32) || defined(_WIN32) || defined(LWS_AMAZON_RTOS)
-		context->max_fds = getdtablesize();
+	context->max_fds = getdtablesize();
 #else
-		context->max_fds = sysconf(_SC_OPEN_MAX);
+	context->max_fds = sysconf(_SC_OPEN_MAX);
 #endif
+	if (context->max_fds < 0) {
+		lwsl_err("%s: problem getting process max files\n",
+			 __func__);
+
+		return NULL;
+	}
 #endif
 
-		if (context->max_fds < 0) {
-			lwsl_err("%s: problem getting process max files\n",
-				 __func__);
-
-			return NULL;
-		}
-
-	if (info->count_threads)
-		context->count_threads = info->count_threads;
-	else
-		context->count_threads = 1;
-
-	if (context->count_threads > LWS_MAX_SMP)
-		context->count_threads = LWS_MAX_SMP;
-
 	/*
 	 * deal with any max_fds override, if it's reducing (setting it to
 	 * more than ulimit -n is meaningless).  The platform init will
 	 * figure out what if this is something it can deal with.
 	 */
 	if (info->fd_limit_per_thread) {
-		int mf = info->fd_limit_per_thread * context->count_threads;
+		int mf = lpf * context->count_threads;
 
 		if (mf < context->max_fds) {
 			context->max_fds_unrelated_to_ulimit = 1;
@@ -283,6 +462,13 @@ lwsl_info("context created\n");
 		goto fail_event_libs;
 #endif
 
+	if (lws_check_opt(context->options, LWS_SERVER_OPTION_GLIB))
+#if defined(LWS_WITH_GLIB)
+		context->event_loop_ops = &event_loop_ops_glib;
+#else
+		goto fail_event_libs;
+#endif
+
 	if (!context->event_loop_ops)
 		goto fail_event_libs;
 
@@ -341,25 +527,40 @@ lwsl_info("context created\n");
 		else
 			context->max_http_header_pool = context->max_fds;
 
+
 	if (info->fd_limit_per_thread)
-		context->fd_limit_per_thread = info->fd_limit_per_thread;
+		context->fd_limit_per_thread = lpf;
 	else
-		context->fd_limit_per_thread = context->max_fds /
-					       context->count_threads;
+		if (context->count_threads)
+			context->fd_limit_per_thread = context->max_fds /
+							context->count_threads;
 
 #if defined(LWS_WITH_NETWORK)
+
+	context->default_retry.retry_ms_table = default_backoff_table;
+	context->default_retry.conceal_count =
+			context->default_retry.retry_ms_table_count =
+					LWS_ARRAY_SIZE(default_backoff_table);
+	context->default_retry.jitter_percent = 20;
+	context->default_retry.secs_since_valid_ping = 300;
+	context->default_retry.secs_since_valid_hangup = 310;
+
+	if (info->retry_and_idle_policy &&
+	    info->retry_and_idle_policy->secs_since_valid_ping) {
+		context->default_retry.secs_since_valid_ping =
+				info->retry_and_idle_policy->secs_since_valid_ping;
+		context->default_retry.secs_since_valid_hangup =
+				info->retry_and_idle_policy->secs_since_valid_hangup;
+	}
+
 	/*
 	 * Allocate the per-thread storage for scratchpad buffers,
 	 * and header data pool
 	 */
+	u = (uint8_t *)&context[1];
 	for (n = 0; n < context->count_threads; n++) {
-		context->pt[n].serv_buf = lws_malloc(
-				context->pt_serv_buf_size + sizeof(struct lws),
-						     "pt_serv_buf");
-		if (!context->pt[n].serv_buf) {
-			lwsl_err("OOM\n");
-			return NULL;
-		}
+		context->pt[n].serv_buf = u;
+		u += context->pt_serv_buf_size;
 
 		context->pt[n].context = context;
 		context->pt[n].tid = n;
@@ -371,8 +572,8 @@ lwsl_info("context created\n");
 		 * when the source of the callback is not actually from a wsi
 		 * context.
 		 */
-		context->pt[n].fake_wsi = (struct lws *)(context->pt[n].serv_buf +
-						context->pt_serv_buf_size);
+		context->pt[n].fake_wsi = (struct lws *)u;
+		u += sizeof(struct lws);
 
 		memset(context->pt[n].fake_wsi, 0, sizeof(struct lws));
 
@@ -384,6 +585,16 @@ lwsl_info("context created\n");
 #if defined(LWS_WITH_SEQUENCER)
 		lws_seq_pt_init(&context->pt[n]);
 #endif
+
+		LWS_FOR_EVERY_AVAILABLE_ROLE_START(ar) {
+			if (ar->pt_init_destroy)
+				ar->pt_init_destroy(context, info,
+						    &context->pt[n], 0);
+		} LWS_FOR_EVERY_AVAILABLE_ROLE_END;
+
+#if defined(LWS_WITH_CGI)
+		role_ops_cgi.pt_init_destroy(context, info, &context->pt[n], 0);
+#endif
 	}
 
 	lwsl_info(" Threads: %d each %d fds\n", context->count_threads,
@@ -435,11 +646,13 @@ lwsl_info("context created\n");
 	}
 	lwsl_info(" mem: pollfd map:      %5u B\n", n);
 #endif
+#if defined(LWS_WITH_SERVER)
 	if (info->server_string) {
 		context->server_string = info->server_string;
 		context->server_string_len = (short)
 				strlen(context->server_string);
 	}
+#endif
 
 #if LWS_MAX_SMP > 1
 	/* each thread serves his own chunk of fds */
@@ -468,40 +681,20 @@ lwsl_info("context created\n");
 				goto bail;
 		}
 
-#if !defined(LWS_AMAZON_RTOS)
 	if (lws_create_event_pipes(context))
 		goto bail;
-#endif
 #endif
 
 	lws_context_init_ssl_library(info);
 
 	context->user_space = info->user;
-#if defined(LWS_WITH_NETWORK)
-	/*
-	 * if he's not saying he'll make his own vhosts later then act
-	 * compatibly and make a default vhost using the data in the info
-	 */
-	if (!lws_check_opt(info->options, LWS_SERVER_OPTION_EXPLICIT_VHOSTS))
-		if (!lws_create_vhost(context, info)) {
-			lwsl_err("Failed to create default vhost\n");
-			for (n = 0; n < context->count_threads; n++)
-				lws_free_set_NULL(context->pt[n].serv_buf);
-#if defined(LWS_WITH_PEER_LIMITS)
-			lws_free_set_NULL(context->pl_hash_table);
-#endif
-			goto fail_clean_pipes;
-		}
 
-	lws_context_init_extensions(info, context);
-
-	lwsl_info(" mem: per-conn:        %5lu bytes + protocol rx buf\n",
-		    (unsigned long)sizeof(struct lws));
-#endif
+#if defined(LWS_WITH_SERVER)
 	strcpy(context->canonical_hostname, "unknown");
 #if defined(LWS_WITH_NETWORK)
 	lws_server_get_canonical_hostname(context, info);
 #endif
+#endif
 
 #if defined(LWS_WITH_STATS)
 	context->pt[0].sul_stats.cb = lws_sul_stats_cb;
@@ -519,6 +712,135 @@ lwsl_info("context created\n");
 	context->count_caps = info->count_caps;
 #endif
 
+
+#if defined(LWS_WITH_NETWORK)
+
+#if defined(LWS_WITH_SYS_ASYNC_DNS) || defined(LWS_WITH_SYS_NTPCLIENT) || \
+	defined(LWS_WITH_SYS_DHCP_CLIENT)
+	{
+		/*
+		 * system vhost
+		 */
+
+		struct lws_context_creation_info ii;
+		const struct lws_protocols *pp[4];
+		struct lws_vhost *vh;
+#if defined(LWS_WITH_SYS_ASYNC_DNS)
+		extern const struct lws_protocols lws_async_dns_protocol;
+#endif
+#if defined(LWS_WITH_SYS_NTPCLIENT)
+		extern const struct lws_protocols lws_system_protocol_ntpc;
+#endif
+#if defined(LWS_WITH_SYS_DHCP_CLIENT)
+		extern const struct lws_protocols lws_system_protocol_dhcpc;
+#endif
+
+		n = 0;
+#if defined(LWS_WITH_SYS_ASYNC_DNS)
+		pp[n++] = &lws_async_dns_protocol;
+#endif
+#if defined(LWS_WITH_SYS_NTPCLIENT)
+		pp[n++] = &lws_system_protocol_ntpc;
+#endif
+#if defined(LWS_WITH_SYS_DHCP_CLIENT)
+		pp[n++] = &lws_system_protocol_dhcpc;
+#endif
+		pp[n] = NULL;
+
+		memset(&ii, 0, sizeof(ii));
+		ii.vhost_name = "system";
+		ii.pprotocols = pp;
+
+		vh = lws_create_vhost(context, &ii);
+		if (!vh) {
+			lwsl_err("%s: failed to create system vhost\n",
+				 __func__);
+			goto bail;
+		}
+
+		context->vhost_system = vh;
+
+		if (lws_protocol_init_vhost(vh, NULL)) {
+			lwsl_err("%s: failed to init system vhost\n", __func__);
+			goto bail;
+		}
+#if defined(LWS_WITH_SYS_ASYNC_DNS)
+		if (lws_async_dns_init(context))
+			goto bail;
+#endif
+	}
+#endif
+
+	/*
+	 * init the lws_state mgr for the system state
+	 */
+#if defined(_DEBUG)
+	context->mgr_system.state_names = system_state_names;
+#endif
+	context->mgr_system.name = "system";
+	context->mgr_system.state = LWS_SYSTATE_CONTEXT_CREATED;
+	context->mgr_system.parent = context;
+
+	context->protocols_notify.name = "prot_init";
+	context->protocols_notify.notify_cb = lws_state_notify_protocol_init;
+
+	lws_state_reg_notifier(&context->mgr_system, &context->protocols_notify);
+
+	/*
+	 * insert user notifiers here so they can participate with vetoing us
+	 * trying to jump straight to operational, or at least observe us
+	 * reaching 'operational', before we returned from context creation.
+	 */
+
+	lws_state_reg_notifier_list(&context->mgr_system,
+				    info->register_notifier_list);
+
+	/*
+	 * if he's not saying he'll make his own vhosts later then act
+	 * compatibly and make a default vhost using the data in the info
+	 */
+	if (!lws_check_opt(info->options, LWS_SERVER_OPTION_EXPLICIT_VHOSTS))
+		if (!lws_create_vhost(context, info)) {
+			lwsl_err("Failed to create default vhost\n");
+
+#if defined(LWS_WITH_PEER_LIMITS)
+			lws_free_set_NULL(context->pl_hash_table);
+#endif
+			goto fail_clean_pipes;
+		}
+
+#if defined(LWS_WITH_SECURE_STREAMS)
+
+	if (context->pss_policies_json) {
+		/*
+		 * You must create your context with the explicit vhosts flag
+		 * in order to use secure streams
+		 */
+		assert(lws_check_opt(info->options,
+		       LWS_SERVER_OPTION_EXPLICIT_VHOSTS));
+
+		if (lws_ss_policy_parse_begin(context))
+			goto bail;
+
+		n = lws_ss_policy_parse(context,
+					(uint8_t *)context->pss_policies_json,
+					strlen(context->pss_policies_json));
+		if (n != LEJP_CONTINUE && n < 0)
+			goto bail;
+
+		if (lws_ss_policy_set(context, "hardcoded")) {
+			lwsl_err("%s: policy set failed\n", __func__);
+			goto bail;
+		}
+	} else
+		lws_create_vhost(context, info);
+#endif
+
+	lws_context_init_extensions(info, context);
+
+	lwsl_info(" mem: per-conn:        %5lu bytes + protocol rx buf\n",
+		    (unsigned long)sizeof(struct lws));
+
 	/*
 	 * drop any root privs for this process
 	 * to listen on port < 1023 we would have needed root, but now we are
@@ -528,7 +850,18 @@ lwsl_info("context created\n");
 		if (lws_plat_drop_app_privileges(context, 1))
 			goto bail;
 
-#if defined(LWS_WITH_NETWORK)
+	/*
+	 * We want to move on the syste, state as far as it can go towards
+	 * OPERATIONAL now.  But we have to return from here first so the user
+	 * code that called us can set its copy of context, which it may be
+	 * relying on to perform operations triggered by the state change.
+	 *
+	 * We set up a sul to come back immediately and do the state change.
+	 */
+
+	lws_sul_schedule(context, 0, &context->sul_system_state,
+			 lws_context_creation_completion_cb, 1);
+
 	/* expedite post-context init (eg, protocols) */
 	lws_cancel_service(context);
 #endif
@@ -570,7 +903,7 @@ fail_event_libs:
 	return NULL;
 }
 
-LWS_VISIBLE LWS_EXTERN int
+int
 lws_context_is_deprecated(struct lws_context *context)
 {
 	return context->deprecated;
@@ -607,10 +940,13 @@ static void
 lws_context_destroy3(struct lws_context *context)
 {
 	struct lws_context **pcontext_finalize = context->pcontext_finalize;
-#if defined(LWS_WITH_NETWORK)
 	int n;
 
-	lwsl_debug("%s\n", __func__);
+#if defined(LWS_WITH_NETWORK)
+
+	context->finalize_destroy_after_internal_loops_stopped = 1;
+	if (context->event_loop_ops->destroy_context2)
+		context->event_loop_ops->destroy_context2(context);
 
 	for (n = 0; n < context->count_threads; n++) {
 		struct lws_context_per_thread *pt = &context->pt[n];
@@ -618,11 +954,18 @@ lws_context_destroy3(struct lws_context *context)
 #if defined(LWS_WITH_SEQUENCER)
 		lws_seq_destroy_all_on_pt(pt);
 #endif
+		LWS_FOR_EVERY_AVAILABLE_ROLE_START(ar) {
+			if (ar->pt_init_destroy)
+				ar->pt_init_destroy(context, NULL, pt, 1);
+		} LWS_FOR_EVERY_AVAILABLE_ROLE_END;
 
+#if defined(LWS_WITH_CGI)
+		role_ops_cgi.pt_init_destroy(context, NULL, pt, 1);
+#endif
+#if 0
 		if (context->event_loop_ops->destroy_pt)
 			context->event_loop_ops->destroy_pt(context, n);
-
-		lws_free_set_NULL(context->pt[n].serv_buf);
+#endif
 
 #if defined(LWS_ROLE_H1) || defined(LWS_ROLE_H2)
 		while (pt->http.ah_list)
@@ -630,11 +973,27 @@ lws_context_destroy3(struct lws_context *context)
 #endif
 	}
 
+#if defined(LWS_WITH_SYS_ASYNC_DNS)
+	lws_async_dns_deinit(&context->async_dns);
+#endif
+#if defined(LWS_WITH_SYS_DHCP_CLIENT)
+	lws_dhcpc_remove(context, NULL);
+#endif
+
 	if (context->pt[0].fds)
 		lws_free_set_NULL(context->pt[0].fds);
 #endif
 	lws_context_deinit_ssl_library(context);
 
+#if defined(LWS_WITH_DETAILED_LATENCIES)
+	if (context->latencies_fd != -1)
+		compatible_close(context->latencies_fd);
+#endif
+
+	for (n = 0; n < LWS_SYSBLOB_TYPE_COUNT; n++)
+		lws_system_blob_destroy(
+				lws_system_get_blob(context, n, 0));
+
 	lws_free(context);
 	lwsl_info("%s: ctx %p freed\n", __func__, context);
 
@@ -651,6 +1010,7 @@ lws_context_destroy2(struct lws_context *context)
 {
 #if defined(LWS_WITH_NETWORK)
 	struct lws_vhost *vh = NULL, *vh1;
+	int n;
 #endif
 #if defined(LWS_WITH_PEER_LIMITS)
 	uint32_t nu;
@@ -662,6 +1022,48 @@ lws_context_destroy2(struct lws_context *context)
 
 	context->being_destroyed2 = 1;
 #if defined(LWS_WITH_NETWORK)
+
+	/*
+	 * We're going to trash things like vhost-protocols
+	 * So we need to finish dealing with wsi close that
+	 * might make callbacks first
+	 */
+	for (n = 0; n < context->count_threads; n++) {
+		struct lws_context_per_thread *pt = &context->pt[n];
+
+		(void)pt;
+
+#if defined(LWS_WITH_SECURE_STREAMS)
+		lws_dll2_foreach_safe(&pt->ss_owner, NULL, lws_ss_destroy_dll);
+		if (context->ac_policy)
+			lwsac_free(&context->ac_policy);
+#endif
+
+#if defined(LWS_WITH_SECURE_STREAMS_PROXY_API)
+		lws_dll2_foreach_safe(&pt->ss_client_owner, NULL, lws_sspc_destroy_dll);
+#endif
+
+#if defined(LWS_WITH_SEQUENCER)
+		lws_seq_destroy_all_on_pt(pt);
+#endif
+		LWS_FOR_EVERY_AVAILABLE_ROLE_START(ar) {
+			if (ar->pt_init_destroy)
+				ar->pt_init_destroy(context, NULL, pt, 1);
+		} LWS_FOR_EVERY_AVAILABLE_ROLE_END;
+
+#if defined(LWS_WITH_CGI)
+		role_ops_cgi.pt_init_destroy(context, NULL, pt, 1);
+#endif
+
+		if (context->event_loop_ops->destroy_pt)
+			context->event_loop_ops->destroy_pt(context, n);
+
+#if defined(LWS_ROLE_H1) || defined(LWS_ROLE_H2)
+		while (pt->http.ah_list)
+			_lws_destroy_ah(pt, pt->http.ah_list);
+#endif
+	}
+
 	/*
 	 * free all the per-vhost allocations
 	 */
@@ -712,13 +1114,11 @@ lws_context_destroy2(struct lws_context *context)
 	lws_check_deferred_free(context, 0, 1);
 #endif
 
-#if LWS_MAX_SMP > 1
-	lws_mutex_refcount_destroy(&context->mr);
-#endif
+	lws_context_unlock(context); /* } context ------ */
+
 #if defined(LWS_WITH_NETWORK)
 	if (context->event_loop_ops->destroy_context2)
 		if (context->event_loop_ops->destroy_context2(context)) {
-			lws_context_unlock(context); /* } context ----------- */
 			context->finalize_destroy_after_internal_loops_stopped = 1;
 			return;
 		}
@@ -730,39 +1130,79 @@ lws_context_destroy2(struct lws_context *context)
 		for (n = 0; n < context->count_threads; n++)
 			if (context->pt[n].inside_service) {
 				lwsl_debug("%p: bailing as inside service\n", __func__);
-				lws_context_unlock(context); /* } context --- */
 				return;
 			}
 	}
 #endif
-	lws_context_unlock(context); /* } context ------------------- */
 
 	lws_context_destroy3(context);
 }
 
+#if defined(LWS_WITH_NETWORK)
+static void
+lws_pt_destroy(struct lws_context_per_thread *pt)
+{
+	volatile struct lws_foreign_thread_pollfd *ftp, *next;
+	volatile struct lws_context_per_thread *vpt;
+
+	assert(!pt->is_destroyed);
+	pt->destroy_self = 0;
+
+	vpt = (volatile struct lws_context_per_thread *)pt;
+	ftp = vpt->foreign_pfd_list;
+	while (ftp) {
+		next = ftp->next;
+		lws_free((void *)ftp);
+		ftp = next;
+	}
+	vpt->foreign_pfd_list = NULL;
+
+	if (pt->pipe_wsi)
+		lws_destroy_event_pipe(pt->pipe_wsi);
+	pt->pipe_wsi = NULL;
+
+	while (pt->fds_count) {
+		struct lws *wsi = wsi_from_fd(pt->context, pt->fds[0].fd);
+
+		if (!wsi)
+			break;
+
+		lws_close_free_wsi(wsi,
+				LWS_CLOSE_STATUS_NOSTATUS_CONTEXT_DESTROY,
+				"ctx destroy"
+				/* no protocol close */);
+	}
+	lws_pt_mutex_destroy(pt);
+
+	pt->is_destroyed = 1;
+
+	lwsl_info("%s: pt destroyed\n", __func__);
+}
+#endif
+
 /*
  * Begin the context takedown
  */
 
-LWS_VISIBLE void
+void
 lws_context_destroy(struct lws_context *context)
 {
 #if defined(LWS_WITH_NETWORK)
-	volatile struct lws_foreign_thread_pollfd *ftp, *next;
-	volatile struct lws_context_per_thread *vpt;
 	struct lws_vhost *vh = NULL;
-	struct lws wsi;
-	int n, m;
+	int m, deferred_pt = 0;
 #endif
 
-	if (!context)
+	if (!context || context->inside_context_destroy)
 		return;
+
+	context->inside_context_destroy = 1;
+
 #if defined(LWS_WITH_NETWORK)
 	if (context->finalize_destroy_after_internal_loops_stopped) {
 		if (context->event_loop_ops->destroy_context2)
 			context->event_loop_ops->destroy_context2(context);
 		lws_context_destroy3(context);
-
+		/* context is invalid, no need to reset inside flag */
 		return;
 	}
 #endif
@@ -776,54 +1216,42 @@ lws_context_destroy(struct lws_context *context)
 			    __func__, context);
 
 		lws_context_destroy3(context);
+		/* context is invalid, no need to reset inside flag */
 		return;
 	}
 
 	lwsl_info("%s: ctx %p\n", __func__, context);
 
 	context->being_destroyed = 1;
-	context->being_destroyed1 = 1;
-	context->requested_kill = 1;
 
 #if defined(LWS_WITH_NETWORK)
+	lws_state_transition(&context->mgr_system, LWS_SYSTATE_POLICY_INVALID);
 	m = context->count_threads;
-	memset(&wsi, 0, sizeof(wsi));
-	wsi.context = context;
-
-#ifdef LWS_LATENCY
-	if (context->worst_latency_info[0])
-		lwsl_notice("Worst latency: %s\n", context->worst_latency_info);
-#endif
 
 	while (m--) {
 		struct lws_context_per_thread *pt = &context->pt[m];
-		vpt = (volatile struct lws_context_per_thread *)pt;
 
-		ftp = vpt->foreign_pfd_list;
-		while (ftp) {
-			next = ftp->next;
-			lws_free((void *)ftp);
-			ftp = next;
+		if (pt->is_destroyed)
+			continue;
+
+		if (pt->inside_lws_service) {
+			pt->destroy_self = 1;
+			deferred_pt = 1;
+			continue;
 		}
-		vpt->foreign_pfd_list = NULL;
 
-		for (n = 0; (unsigned int)n < context->pt[m].fds_count; n++) {
-			struct lws *wsi = wsi_from_fd(context, pt->fds[n].fd);
-			if (!wsi)
-				continue;
-
-			if (wsi->event_pipe)
-				lws_destroy_event_pipe(wsi);
-			else
-				lws_close_free_wsi(wsi,
-					LWS_CLOSE_STATUS_NOSTATUS_CONTEXT_DESTROY,
-					"ctx destroy"
-					/* no protocol close */);
-			n--;
-		}
-		lws_pt_mutex_destroy(pt);
+		lws_pt_destroy(pt);
 	}
 
+	if (deferred_pt) {
+		lwsl_info("%s: waiting for deferred pt close\n", __func__);
+		lws_cancel_service(context);
+		goto out;
+	}
+
+	context->being_destroyed1 = 1;
+	context->requested_kill = 1;
+
 	/*
 	 * inform all the protocols that they are done and will have no more
 	 * callbacks.
@@ -863,11 +1291,11 @@ lws_context_destroy(struct lws_context *context)
 	if (context->event_loop_ops->destroy_context1) {
 		context->event_loop_ops->destroy_context1(context);
 
-		return;
+		goto out;
 	}
 #endif
 
-#if defined(LWS_WITH_ESP32)
+#if defined(LWS_PLAT_FREERTOS)
 #if defined(LWS_AMAZON_RTOS)
 	context->last_free_heap = xPortGetFreeHeapSize();
 #else
@@ -875,6 +1303,23 @@ lws_context_destroy(struct lws_context *context)
 #endif
 #endif
 
+	context->inside_context_destroy = 0;
 	lws_context_destroy2(context);
+
+	return;
+
+#if defined(LWS_WITH_NETWORK)
+out:
+	context->inside_context_destroy = 0;
+#endif
 }
 
+struct lws_context *
+lws_system_context_from_system_mgr(lws_state_manager_t *mgr)
+{
+#if defined(LWS_WITH_NETWORK)
+	return lws_container_of(mgr, struct lws_context, mgr_system);
+#else
+	return NULL;
+#endif
+}
