@@ -1,25 +1,28 @@
 /*
  * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2010-2019 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2019 Andy Green <andy@warmcat.com>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation:
- *  version 2.1 of the License.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA  02110-1301  USA
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
  */
 
-#include "core/private.h"
+#include "private-lib-core.h"
 
 #ifdef LWS_HAVE_SYS_TYPES_H
 #include <sys/types.h>
@@ -38,38 +41,27 @@ static void (*lwsl_emit)(int level, const char *line)
 #endif
 	;
 #ifndef LWS_PLAT_OPTEE
-static const char * const log_level_names[] = {
-	"E",
-	"W",
-	"N",
-	"I",
-	"D",
-	"P",
-	"H",
-	"EXT",
-	"C",
-	"L",
-	"U",
-	"T",
-	"?",
-	"?"
-};
+static const char * log_level_names ="EWNIDPHXCLUT??";
 #endif
 
-LWS_VISIBLE int
+#if defined(LWS_LOGS_TIMESTAMP)
+int
 lwsl_timestamp(int level, char *p, int len)
 {
 #ifndef LWS_PLAT_OPTEE
-#ifndef _WIN32_WCE
-	time_t o_now = time(NULL);
-#endif
+	time_t o_now;
 	unsigned long long now;
+	struct timeval tv;
 	struct tm *ptm = NULL;
 #ifndef WIN32
 	struct tm tm;
 #endif
 	int n;
 
+	gettimeofday(&tv, NULL);
+	o_now = tv.tv_sec;
+	now = ((unsigned long long)tv.tv_sec * 10000) + (tv.tv_usec / 100);
+
 #ifndef _WIN32_WCE
 #ifdef WIN32
 	ptm = localtime(&o_now);
@@ -82,10 +74,10 @@ lwsl_timestamp(int level, char *p, int len)
 	for (n = 0; n < LLL_COUNT; n++) {
 		if (level != (1 << n))
 			continue;
-		now = lws_now_usecs() / 100;
+
 		if (ptm)
 			n = lws_snprintf(p, len,
-				"[%04d/%02d/%02d %02d:%02d:%02d:%04d] %s: ",
+				"[%04d/%02d/%02d %02d:%02d:%02d:%04d] %c: ",
 				ptm->tm_year + 1900,
 				ptm->tm_mon + 1,
 				ptm->tm_mday,
@@ -94,7 +86,7 @@ lwsl_timestamp(int level, char *p, int len)
 				ptm->tm_sec,
 				(int)(now % 10000), log_level_names[n]);
 		else
-			n = lws_snprintf(p, len, "[%llu:%04d] %s: ",
+			n = lws_snprintf(p, len, "[%llu:%04d] %c: ",
 					(unsigned long long) now / 10000,
 					(int)(now % 10000), log_level_names[n]);
 		return n;
@@ -105,6 +97,7 @@ lwsl_timestamp(int level, char *p, int len)
 
 	return 0;
 }
+#endif
 
 #ifndef LWS_PLAT_OPTEE
 static const char * const colours[] = {
@@ -118,21 +111,26 @@ static const char * const colours[] = {
 	"[33m", /* LLL_EXT */
 	"[33m", /* LLL_CLIENT */
 	"[33;1m", /* LLL_LATENCY */
-	"[30;1m", /* LLL_USER */
+        "[0;1m", /* LLL_USER */
 	"[31m", /* LLL_THREAD */
 };
 
 static char tty;
 
-LWS_VISIBLE void
-lwsl_emit_stderr(int level, const char *line)
+static void
+_lwsl_emit_stderr(int level, const char *line, int ts)
 {
 	char buf[50];
 	int n, m = LWS_ARRAY_SIZE(colours) - 1;
 
 	if (!tty)
 		tty = isatty(2) | 2;
-	lwsl_timestamp(level, buf, sizeof(buf));
+
+	buf[0] = '\0';
+#if defined(LWS_LOGS_TIMESTAMP)
+	if (ts)
+		lwsl_timestamp(level, buf, sizeof(buf));
+#endif
 
 	if (tty == 3) {
 		n = 1 << (LWS_ARRAY_SIZE(colours) - 1);
@@ -147,33 +145,28 @@ lwsl_emit_stderr(int level, const char *line)
 		fprintf(stderr, "%s%s", buf, line);
 }
 
-LWS_VISIBLE void
+void
+lwsl_emit_stderr(int level, const char *line)
+{
+	_lwsl_emit_stderr(level, line, 1);
+}
+
+void
 lwsl_emit_stderr_notimestamp(int level, const char *line)
 {
-	int n, m = LWS_ARRAY_SIZE(colours) - 1;
-
-	if (!tty)
-		tty = isatty(2) | 2;
-
-	if (tty == 3) {
-		n = 1 << (LWS_ARRAY_SIZE(colours) - 1);
-		while (n) {
-			if (level & n)
-				break;
-			m--;
-			n >>= 1;
-		}
-		fprintf(stderr, "%c%s%s%c[0m", 27, colours[m], line, 27);
-	} else
-		fprintf(stderr, "%s", line);
+	_lwsl_emit_stderr(level, line, 0);
 }
 
 #endif
 
 #if !(defined(LWS_PLAT_OPTEE) && !defined(LWS_WITH_NETWORK))
-LWS_VISIBLE void _lws_logv(int filter, const char *format, va_list vl)
+void _lws_logv(int filter, const char *format, va_list vl)
 {
+#if LWS_MAX_SMP == 1
 	static char buf[256];
+#else
+	char buf[1024];
+#endif
 	int n;
 
 	if (!(log_level & filter))
@@ -195,7 +188,7 @@ LWS_VISIBLE void _lws_logv(int filter, const char *format, va_list vl)
 	lwsl_emit(filter, buf);
 }
 
-LWS_VISIBLE void _lws_log(int filter, const char *format, ...)
+void _lws_log(int filter, const char *format, ...)
 {
 	va_list ap;
 
@@ -204,20 +197,19 @@ LWS_VISIBLE void _lws_log(int filter, const char *format, ...)
 	va_end(ap);
 }
 #endif
-LWS_VISIBLE void lws_set_log_level(int level,
-				   void (*func)(int level, const char *line))
+void lws_set_log_level(int level, void (*func)(int level, const char *line))
 {
 	log_level = level;
 	if (func)
 		lwsl_emit = func;
 }
 
-LWS_VISIBLE int lwsl_visible(int level)
+int lwsl_visible(int level)
 {
 	return log_level & level;
 }
 
-LWS_VISIBLE void
+void
 lwsl_hexdump_level(int hexdump_level, const void *vbuf, size_t len)
 {
 	unsigned char *buf = (unsigned char *)vbuf;
@@ -232,8 +224,7 @@ lwsl_hexdump_level(int hexdump_level, const void *vbuf, size_t len)
 	}
 
 	if (!vbuf) {
-		_lws_log(hexdump_level, "(hexdump: trying to dump %d at NULL)\n",
-					(int)len);
+		_lws_log(hexdump_level, "(hexdump: NULL ptr)\n");
 		return;
 	}
 
@@ -270,7 +261,7 @@ lwsl_hexdump_level(int hexdump_level, const void *vbuf, size_t len)
 	_lws_log(hexdump_level, "\n");
 }
 
-LWS_VISIBLE void
+void
 lwsl_hexdump(const void *vbuf, size_t len)
 {
 #if defined(_DEBUG)
