@@ -1,25 +1,28 @@
 /*
  * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2010-2018 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2019 Andy Green <andy@warmcat.com>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation:
- *  version 2.1 of the License.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA  02110-1301  USA
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
  */
 
-#include "core/private.h"
+#include "private-lib-core.h"
 
 static void
 lws_event_hrtimer_cb(int fd, short event, void *p)
@@ -45,6 +48,9 @@ lws_event_idle_timer_cb(int fd, short event, void *p)
 	struct timeval tv;
 	lws_usec_t us;
 
+	if (pt->is_destroyed)
+		return;
+
 	lws_service_do_ripe_rxflow(pt);
 
 	/*
@@ -77,6 +83,10 @@ lws_event_idle_timer_cb(int fd, short event, void *p)
 		evtimer_add(pt->event.hrtimer, &tv);
 	}
 	lws_pt_unlock(pt);
+
+
+	if (pt->destroy_self)
+		lws_context_destroy(pt->context);
 }
 
 static void
@@ -114,13 +124,20 @@ lws_event_cb(evutil_socket_t sock_fd, short revents, void *ctx)
 	}
 
 	wsi = wsi_from_fd(context, sock_fd);
-	if (!wsi) {
+	if (!wsi)
 		return;
-	}
+
 	pt = &context->pt[(int)wsi->tsi];
+	if (pt->is_destroyed)
+		return;
 
 	lws_service_fd_tsi(context, &eventfd, wsi->tsi);
 
+	if (pt->destroy_self) {
+		lws_context_destroy(pt->context);
+		return;
+	}
+
 	/* set the idle timer for 1ms ahead */
 
 	tv.tv_sec = 0;
@@ -128,7 +145,7 @@ lws_event_cb(evutil_socket_t sock_fd, short revents, void *ctx)
 	evtimer_add(pt->event.idle_timer, &tv);
 }
 
-LWS_VISIBLE void
+void
 lws_event_sigint_cb(evutil_socket_t sock_fd, short revents, void *ctx)
 {
 	struct lws_context_per_thread *pt = ctx;
@@ -180,6 +197,7 @@ elops_init_pt_event(struct lws_context *context, void *_loop, int tsi)
 					(EV_READ | EV_PERSIST), lws_event_cb,
 					&vh->lserv_wsi->w_read);
 			event_add(vh->lserv_wsi->w_read.event.watcher, NULL);
+			vh->lserv_wsi->w_read.event.set = 1;
 		}
 		vh = vh->vhost_next;
 	}
@@ -249,24 +267,33 @@ elops_io_event(struct lws *wsi, int flags)
 {
 	struct lws_context_per_thread *pt = &wsi->context->pt[(int)wsi->tsi];
 
-	if (!pt->event.io_loop || wsi->context->being_destroyed)
+	if (!pt->event.io_loop || wsi->context->being_destroyed ||
+	    pt->is_destroyed)
 		return;
 
 	assert((flags & (LWS_EV_START | LWS_EV_STOP)) &&
 	       (flags & (LWS_EV_READ | LWS_EV_WRITE)));
 
 	if (flags & LWS_EV_START) {
-		if (flags & LWS_EV_WRITE)
+		if ((flags & LWS_EV_WRITE) && !wsi->w_write.event.set) {
 			event_add(wsi->w_write.event.watcher, NULL);
+			wsi->w_write.event.set = 1;
+		}
 
-		if (flags & LWS_EV_READ)
+		if ((flags & LWS_EV_READ) && !wsi->w_read.event.set) {
 			event_add(wsi->w_read.event.watcher, NULL);
+			wsi->w_read.event.set = 1;
+		}
 	} else {
-		if (flags & LWS_EV_WRITE)
+		if ((flags & LWS_EV_WRITE) && wsi->w_write.event.set) {
 			event_del(wsi->w_write.event.watcher);
+			wsi->w_write.event.set = 0;
+		}
 
-		if (flags & LWS_EV_READ)
+		if ((flags & LWS_EV_READ) && wsi->w_read.event.set) {
 			event_del(wsi->w_read.event.watcher);
+			wsi->w_read.event.set = 0;
+		}
 	}
 }
 
@@ -308,22 +335,42 @@ elops_destroy_pt_event(struct lws_context *context, int tsi)
 	if (!pt->event_loop_foreign) {
 		event_del(pt->w_sigint.event.watcher);
 		event_free(pt->w_sigint.event.watcher);
-
-		event_base_free(pt->event.io_loop);
+		event_base_loopexit(pt->event.io_loop, NULL);
+	//	event_base_free(pt->event.io_loop);
+	//	pt->event.io_loop = NULL;
+		lwsl_notice("%s: set to exit loop\n", __func__);
 	}
 }
 
 static void
 elops_destroy_wsi_event(struct lws *wsi)
 {
+	struct lws_context_per_thread *pt;
+
 	if (!wsi)
 		return;
 
-	if (wsi->w_read.event.watcher)
-		event_free(wsi->w_read.event.watcher);
+	pt = &wsi->context->pt[(int)wsi->tsi];
+	if (pt->is_destroyed)
+		return;
 
-	if (wsi->w_write.event.watcher)
+	if (wsi->w_read.event.watcher) {
+		event_free(wsi->w_read.event.watcher);
+		wsi->w_read.event.watcher = NULL;
+	}
+
+	if (wsi->w_write.event.watcher) {
 		event_free(wsi->w_write.event.watcher);
+		wsi->w_write.event.watcher = NULL;
+	}
+}
+
+static int
+elops_wsi_logical_close_event(struct lws *wsi)
+{
+	elops_destroy_wsi_event(wsi);
+
+	return 0;
 }
 
 static int
@@ -392,7 +439,9 @@ elops_destroy_context2_event(struct lws_context *context)
 		} else
 			lwsl_debug("%s: %d: everything closed OK\n", __func__, n);
 #endif
+		lwsl_err("%s: event_base_free\n", __func__);
 		event_base_free(pt->event.io_loop);
+		pt->event.io_loop = NULL;
 
 	}
 
@@ -408,7 +457,7 @@ struct lws_event_loop_ops event_loop_ops_event = {
 	/* destroy_context2 */		elops_destroy_context2_event,
 	/* init_vhost_listen_wsi */	elops_init_vhost_listen_wsi_event,
 	/* init_pt */			elops_init_pt_event,
-	/* wsi_logical_close */		NULL,
+	/* wsi_logical_close */		elops_wsi_logical_close_event,
 	/* check_client_connect_ok */	NULL,
 	/* close_handle_manually */	NULL,
 	/* accept */			elops_accept_event,
@@ -417,5 +466,5 @@ struct lws_event_loop_ops event_loop_ops_event = {
 	/* destroy_pt */		elops_destroy_pt_event,
 	/* destroy wsi */		elops_destroy_wsi_event,
 
-	/* periodic_events_available */	0,
+	/* flags */			0,
 };
