@@ -1,25 +1,28 @@
 /*
  * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2010-2018 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2019 Andy Green <andy@warmcat.com>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation:
- *  version 2.1 of the License.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA  02110-1301  USA
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
  */
 
-#include "core/private.h"
+#include "private-lib-core.h"
 
 static void
 lws_uv_sultimer_cb(uv_timer_t *timer
@@ -81,6 +84,9 @@ lws_io_cb(uv_poll_t *watcher, int status, int revents)
 	struct lws_context_per_thread *pt = &context->pt[(int)wsi->tsi];
 	struct lws_pollfd eventfd;
 
+	if (pt->is_destroyed)
+		return;
+
 #if defined(WIN32) || defined(_WIN32)
 	eventfd.fd = watcher->socket;
 #else
@@ -113,6 +119,11 @@ lws_io_cb(uv_poll_t *watcher, int status, int revents)
 	}
 	lws_service_fd_tsi(context, &eventfd, wsi->tsi);
 
+	if (pt->destroy_self) {
+		lws_context_destroy(pt->context);
+		return;
+	}
+
 	uv_idle_start(&pt->uv.idle, lws_uv_idle);
 }
 
@@ -236,7 +247,7 @@ lws_uv_close_cb_sa(uv_handle_t *handle)
  * .... when the libuv object is created...
  */
 
-LWS_VISIBLE void
+void
 lws_libuv_static_refcount_add(uv_handle_t *h, struct lws_context *context)
 {
 	LWS_UV_REFCOUNT_STATIC_HANDLE_NEW(h, context);
@@ -246,7 +257,7 @@ lws_libuv_static_refcount_add(uv_handle_t *h, struct lws_context *context)
  * ... and in the close callback when the object is closed.
  */
 
-LWS_VISIBLE void
+void
 lws_libuv_static_refcount_del(uv_handle_t *h)
 {
 	lws_uv_close_cb_sa(h);
@@ -263,14 +274,14 @@ static void lws_uv_walk_cb(uv_handle_t *handle, void *arg)
 		uv_close(handle, lws_uv_close_cb);
 }
 
-LWS_VISIBLE void
+void
 lws_close_all_handles_in_loop(uv_loop_t *loop)
 {
 	uv_walk(loop, lws_uv_walk_cb, NULL);
 }
 
 
-LWS_VISIBLE void
+void
 lws_libuv_stop_without_kill(const struct lws_context *context, int tsi)
 {
 	if (context->pt[tsi].uv.io_loop)
@@ -279,7 +290,7 @@ lws_libuv_stop_without_kill(const struct lws_context *context, int tsi)
 
 
 
-LWS_VISIBLE uv_loop_t *
+uv_loop_t *
 lws_uv_getloop(struct lws_context *context, int tsi)
 {
 	if (context->pt[tsi].uv.io_loop)
@@ -364,8 +375,8 @@ lws_uv_plugins_init(struct lws_context *context, const char * const *d)
 #endif
 			if (uv_dlsym(&lib, path, &v)) {
 				uv_dlerror(&lib);
-				lwsl_err("Failed to get %s on %s: %s", path,
-						dent.name, lib.errmsg);
+				lwsl_err("%s: Failed to get '%s' on %s: %s\n",
+					 __func__, path, dent.name, lib.errmsg);
 				uv_dlclose(&lib);
 				goto bail;
 			}
@@ -494,7 +505,7 @@ elops_destroy_context1_uv(struct lws_context *context)
 						  UV_RUN_NOWAIT)))
 					;
 			if (m)
-				lwsl_err("%s: tsi %d: not all closed\n",
+				lwsl_info("%s: tsi %d: not all closed\n",
 					 __func__, n);
 
 		}
@@ -611,7 +622,7 @@ elops_accept_uv(struct lws *wsi)
 
 	if (wsi->role_ops->file_handle)
 		uv_poll_init(pt->uv.io_loop, wsi->w_read.uv.pwatcher,
-			     (int)(long long)wsi->desc.filefd);
+			     (int)(lws_intptr_t)wsi->desc.filefd);
 	else
 		uv_poll_init_socket(pt->uv.io_loop,
 				    wsi->w_read.uv.pwatcher,
@@ -769,7 +780,7 @@ elops_destroy_pt_uv(struct lws_context *context, int tsi)
  * called again to bind the vhost
  */
 
-LWS_VISIBLE int
+int
 elops_init_pt_uv(struct lws_context *context, void *_loop, int tsi)
 {
 	struct lws_context_per_thread *pt = &context->pt[tsi];
@@ -848,7 +859,7 @@ lws_libuv_closewsi(uv_handle_t* handle)
 	struct lws *wsi = (struct lws *)handle->data;
 	struct lws_context *context = lws_get_context(wsi);
 	struct lws_context_per_thread *pt = &context->pt[(int)wsi->tsi];
-#if !defined(LWS_WITHOUT_SERVER)
+#if defined(LWS_WITH_SERVER)
 	int lspd = 0;
 #endif
 
@@ -858,7 +869,7 @@ lws_libuv_closewsi(uv_handle_t* handle)
 	 * We get called back here for every wsi that closes
 	 */
 
-#if !defined(LWS_WITHOUT_SERVER)
+#if defined(LWS_WITH_SERVER)
 	if (wsi->role_ops == &role_ops_listen && wsi->context->deprecated) {
 		lspd = 1;
 		context->deprecation_pending_listen_close_count--;
@@ -874,7 +885,7 @@ lws_libuv_closewsi(uv_handle_t* handle)
 	/* it's our job to close the handle finally */
 	lws_free(handle);
 
-#if !defined(LWS_WITHOUT_SERVER)
+#if defined(LWS_WITH_SERVER)
 	if (lspd == 2 && context->deprecation_cb) {
 		lwsl_notice("calling deprecation callback\n");
 		context->deprecation_cb();
@@ -965,5 +976,5 @@ struct lws_event_loop_ops event_loop_ops_uv = {
 	/* destroy_pt */		elops_destroy_pt_uv,
 	/* destroy wsi */		NULL,
 
-	/* periodic_events_available */	0,
+	/* flags */			0,
 };
