@@ -1,26 +1,29 @@
 /*
- * libwebsockets - JSON Web Signature support
+ * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2017 - 2018 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2019 Andy Green <andy@warmcat.com>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation:
- *  version 2.1 of the License.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA  02110-1301  USA
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
  */
 
-#include "core/private.h"
-#include "private.h"
+#include "private-lib-core.h"
+#include "private-lib-jose-jws.h"
 
 /*
  * Currently only support flattened or compact (implicitly single signature)
@@ -157,7 +160,7 @@ lws_jws_json_parse(struct lws_jws *jws, const uint8_t *buf, int len,
 	return 0;
 }
 
-LWS_VISIBLE void
+void
 lws_jws_init(struct lws_jws *jws, struct lws_jwk *jwk,
 	     struct lws_context *context)
 {
@@ -178,14 +181,14 @@ lws_jws_map_bzero(struct lws_jws_map *map)
 			lws_explicit_bzero((void *)map->buf[n], map->len[n]);
 }
 
-LWS_VISIBLE void
+void
 lws_jws_destroy(struct lws_jws *jws)
 {
 	lws_jws_map_bzero(&jws->map);
 	jws->jwk = NULL;
 }
 
-LWS_VISIBLE int
+int
 lws_jws_dup_element(struct lws_jws_map *map, int idx, char *temp, int *temp_len,
 		    const void *in, size_t in_len, size_t actual_alloc)
 {
@@ -205,7 +208,7 @@ lws_jws_dup_element(struct lws_jws_map *map, int idx, char *temp, int *temp_len,
 	return 0;
 }
 
-LWS_VISIBLE int
+int
 lws_jws_encode_b64_element(struct lws_jws_map *map, int idx,
 			   char *temp, int *temp_len, const void *in,
 			   size_t in_len)
@@ -227,7 +230,7 @@ lws_jws_encode_b64_element(struct lws_jws_map *map, int idx,
 	return 0;
 }
 
-LWS_VISIBLE int
+int
 lws_jws_randomize_element(struct lws_context *context, struct lws_jws_map *map,
 			  int idx, char *temp, int *temp_len, size_t random_len,
 			  size_t actual_alloc)
@@ -241,7 +244,7 @@ lws_jws_randomize_element(struct lws_context *context, struct lws_jws_map *map,
 	map->len[idx] = random_len;
 	map->buf[idx] = temp;
 
-	if (lws_get_random(context, temp, random_len) != (int)random_len) {
+	if (lws_get_random(context, temp, random_len) != random_len) {
 		lwsl_err("Problem getting random\n");
 		return -1;
 	}
@@ -251,7 +254,7 @@ lws_jws_randomize_element(struct lws_context *context, struct lws_jws_map *map,
 	return 0;
 }
 
-LWS_VISIBLE int
+int
 lws_jws_alloc_element(struct lws_jws_map *map, int idx, char *temp,
 		      int *temp_len, size_t len, size_t actual_alloc)
 {
@@ -268,7 +271,7 @@ lws_jws_alloc_element(struct lws_jws_map *map, int idx, char *temp,
 	return 0;
 }
 
-LWS_VISIBLE int
+int
 lws_jws_base64_enc(const char *in, size_t in_len, char *out, size_t out_max)
 {
 	int n;
@@ -289,7 +292,7 @@ lws_jws_base64_enc(const char *in, size_t in_len, char *out, size_t out_max)
 	return n;
 }
 
-LWS_VISIBLE int
+int
 lws_jws_b64_compact_map(const char *in, int len, struct lws_jws_map *map)
 {
 	int me = 0;
@@ -317,7 +320,7 @@ lws_jws_b64_compact_map(const char *in, int len, struct lws_jws_map *map)
  * map_b64 set to b64 elements
  */
 
-LWS_VISIBLE int
+int
 lws_jws_compact_decode(const char *in, int len, struct lws_jws_map *map,
 		       struct lws_jws_map *map_b64, char *out,
 		       int *out_len)
@@ -384,7 +387,7 @@ lws_jws_compact_decode_map(struct lws_jws_map *map_b64, struct lws_jws_map *map,
 	return 0;
 }
 
-LWS_VISIBLE int
+int
 lws_jws_encode_section(const char *in, size_t in_len, int first, char **p,
 		       char *end)
 {
@@ -406,7 +409,7 @@ lws_jws_encode_section(const char *in, size_t in_len, int first, char **p,
 	return (*p) - p_entry;
 }
 
-LWS_VISIBLE int
+int
 lws_jws_compact_encode(struct lws_jws_map *map_b64, /* b64-encoded */
 		       const struct lws_jws_map *map,	/* non-b64 */
 		       char *buf, int *len)
@@ -438,7 +441,7 @@ lws_jws_compact_encode(struct lws_jws_map *map_b64, /* b64-encoded */
  * the JOSE header and signature, decoded versions too.
  */
 
-LWS_VISIBLE int
+int
 lws_jws_sig_confirm(struct lws_jws_map *map_b64, struct lws_jws_map *map,
 		    struct lws_jwk *jwk, struct lws_context *context)
 {
@@ -513,7 +516,7 @@ lws_jws_sig_confirm(struct lws_jws_map *map_b64, struct lws_jws_map *map,
 
 			return -1;
 		}
-		h_len = lws_genhash_size(jose.alg->hash_type);
+		// h_len = lws_genhash_size(jose.alg->hash_type);
 
 		if (lws_genrsa_create(&rsactx, jwk->e, context, padding,
 				LWS_GENHASH_TYPE_UNKNOWN)) {
@@ -663,7 +666,7 @@ lws_jws_sig_confirm(struct lws_jws_map *map_b64, struct lws_jws_map *map,
 
 /* it's already a b64 map, we will make a temp plain version */
 
-LWS_VISIBLE int
+int
 lws_jws_sig_confirm_compact_b64_map(struct lws_jws_map *map_b64,
 				    struct lws_jwk *jwk,
 			            struct lws_context *context,
@@ -684,7 +687,7 @@ lws_jws_sig_confirm_compact_b64_map(struct lws_jws_map *map_b64,
  * plain version
  */
 
-LWS_VISIBLE int
+int
 lws_jws_sig_confirm_compact_b64(const char *in, size_t len,
 				struct lws_jws_map *map, struct lws_jwk *jwk,
 				struct lws_context *context,
@@ -705,7 +708,7 @@ lws_jws_sig_confirm_compact_b64(const char *in, size_t len,
 
 /* it's already plain, we will make a temp b64 version */
 
-LWS_VISIBLE int
+int
 lws_jws_sig_confirm_compact(struct lws_jws_map *map, struct lws_jwk *jwk,
 			    struct lws_context *context, char *temp,
 			    int *temp_len)
@@ -884,7 +887,7 @@ lws_jws_sign_from_b64(struct lws_jose *jose, struct lws_jws *jws,
  *   }
  */
 
-LWS_VISIBLE int
+int
 lws_jws_write_flattened_json(struct lws_jws *jws, char *flattened, size_t len)
 {
 	size_t n = 0;
@@ -892,25 +895,34 @@ lws_jws_write_flattened_json(struct lws_jws *jws, char *flattened, size_t len)
 	if (len < 1)
 		return 1;
 
-	n += lws_snprintf(flattened + n, len - n , "{\"payload\": \"%.*s\",\n",
-			  jws->map_b64.len[LJWS_PYLD],
-			  jws->map_b64.buf[LJWS_PYLD]);
+	n += lws_snprintf(flattened + n, len - n , "{\"payload\": \"");
+	lws_strnncpy(flattened + n, jws->map_b64.buf[LJWS_PYLD],
+			jws->map_b64.len[LJWS_PYLD], len - n);
+	n += strlen(flattened + n);
 
-	n += lws_snprintf(flattened + n, len - n , " \"protected\": \"%.*s\",\n",
-			  jws->map_b64.len[LJWS_JOSE],
-			  jws->map_b64.buf[LJWS_JOSE]);
+	n += lws_snprintf(flattened + n, len - n , "\",\n \"protected\": \"");
+	lws_strnncpy(flattened + n, jws->map_b64.buf[LJWS_JOSE],
+			jws->map_b64.len[LJWS_JOSE], len - n);
+	n += strlen(flattened + n);
 
-	if (jws->map_b64.buf[LJWS_UHDR])
-		n += lws_snprintf(flattened + n, len - n , " \"header\": %.*s,\n",
-			  jws->map_b64.len[LJWS_UHDR], jws->map_b64.buf[LJWS_UHDR]);
+	if (jws->map_b64.buf[LJWS_UHDR]) {
+		n += lws_snprintf(flattened + n, len - n , "\",\n \"header\": ");
+		lws_strnncpy(flattened + n, jws->map_b64.buf[LJWS_UHDR],
+				jws->map_b64.len[LJWS_UHDR], len - n);
+		n += strlen(flattened + n);
+	}
 
-	n += lws_snprintf(flattened + n, len - n , " \"signature\": \"%.*s\"}\n",
-			jws->map_b64.len[LJWS_SIG], jws->map_b64.buf[LJWS_SIG]);
+	n += lws_snprintf(flattened + n, len - n , "\",\n \"signature\": \"");
+	lws_strnncpy(flattened + n, jws->map_b64.buf[LJWS_SIG],
+			jws->map_b64.len[LJWS_SIG], len - n);
+	n += strlen(flattened + n);
+
+	n += lws_snprintf(flattened + n, len - n , "\"}\n");
 
 	return (n >= len - 1);
 }
 
-LWS_VISIBLE int
+int
 lws_jws_write_compact(struct lws_jws *jws, char *compact, size_t len)
 {
 	size_t n = 0;
@@ -918,12 +930,15 @@ lws_jws_write_compact(struct lws_jws *jws, char *compact, size_t len)
 	if (len < 1)
 		return 1;
 
-	n += lws_snprintf(compact + n, len - n , "%.*s",
-			  jws->map_b64.len[LJWS_JOSE], jws->map_b64.buf[LJWS_JOSE]);
-	n += lws_snprintf(compact + n, len - n , ".%.*s",
-			  jws->map_b64.len[LJWS_PYLD], jws->map_b64.buf[LJWS_PYLD]);
-	n += lws_snprintf(compact + n, len - n , ".%.*s",
-			  jws->map_b64.len[LJWS_SIG], jws->map_b64.buf[LJWS_SIG]);
+	lws_strnncpy(compact + n, jws->map_b64.buf[LJWS_JOSE],
+		     jws->map_b64.len[LJWS_JOSE], len - n);
+	n += strlen(compact + n);
+	lws_strnncpy(compact + n, jws->map_b64.buf[LJWS_PYLD],
+		     jws->map_b64.len[LJWS_PYLD], len - n);
+	n += strlen(compact + n);
+	lws_strnncpy(compact + n, jws->map_b64.buf[LJWS_SIG],
+		     jws->map_b64.len[LJWS_SIG], len - n);
+	n += strlen(compact + n);
 
 	return n >= len - 1;
 }
