@@ -1,26 +1,29 @@
 /*
- * Lightweight Embedded JSON Parser
+ * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2013-2018 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2019 Andy Green <andy@warmcat.com>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation:
- *  version 2.1 of the License.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA  02110-1301  USA
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
  */
 
 #include <libwebsockets.h>
-#include "core/private.h"
+#include "private-lib-core.h"
 #include <string.h>
 #include <stdio.h>
 
@@ -143,7 +146,8 @@ void
 lejp_check_path_match(struct lejp_ctx *ctx)
 {
 	const char *p, *q;
-	int n, s = sizeof(char *);
+	int n;
+	size_t s = sizeof(char *);
 
 	if (ctx->path_stride)
 		s = ctx->path_stride;
@@ -412,7 +416,7 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 			break;
 
 		case LEJP_MP_VALUE:
-			if (c >= '0' && c <= '9') {
+			if (c == '-' || (c >= '0' && c <= '9')) {
 				ctx->npos = 0;
 				ctx->dcount = 0;
 				ctx->f = 0;
