@@ -1,25 +1,28 @@
 /*
  * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2010-2018 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2019 Andy Green <andy@warmcat.com>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation:
- *  version 2.1 of the License.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA  02110-1301  USA
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
  */
 
-#include <core/private.h>
+#include <private-lib-core.h>
 
 #ifndef min
 #define min(a, b) ((a) < (b) ? (a) : (b))
@@ -67,7 +70,7 @@ lws_read_h1(struct lws *wsi, unsigned char *buf, lws_filepos_t len)
 			assert(0);
 		}
 		lwsl_parser("issuing %d bytes to parser\n", (int)len);
-#if defined(LWS_ROLE_WS) && !defined(LWS_NO_CLIENT)
+#if defined(LWS_ROLE_WS) && defined(LWS_WITH_CLIENT)
 		if (lws_ws_handshake_client(wsi, &buf, (size_t)len))
 			goto bail;
 #endif
@@ -77,8 +80,12 @@ lws_read_h1(struct lws *wsi, unsigned char *buf, lws_filepos_t len)
 			goto bail;
 
 		/* we might have transitioned to RAW */
-		if (wsi->role_ops == &role_ops_raw_skt ||
-		    wsi->role_ops == &role_ops_raw_file)
+		if (wsi->role_ops == &role_ops_raw_skt
+#if defined(LWS_ROLE_RAW_FILE)
+				||
+		    wsi->role_ops == &role_ops_raw_file
+#endif
+		    )
 			 /* we gave the read buffer to RAW handler already */
 			goto read_ok;
 
@@ -137,7 +144,7 @@ http_postbody:
 				struct lws_cgi_args args;
 
 				args.ch = LWS_STDIN;
-				args.stdwsi = &wsi->http.cgi->stdwsi[0];
+				args.stdwsi = &wsi->http.cgi->lsp->stdwsi[0];
 				args.data = buf;
 				args.len = body_chunk_len;
 
@@ -187,7 +194,7 @@ postbody_completion:
 			if (!wsi->http.cgi)
 #endif
 			{
-#if !defined(LWS_NO_SERVER)
+#if defined(LWS_WITH_SERVER)
 				if (lwsi_state(wsi) == LRS_DISCARD_BODY) {
 					/*
 					 * repeat the transaction completed
@@ -209,7 +216,7 @@ postbody_completion:
 				if (n)
 					goto bail;
 
-				if (wsi->http2_substream)
+				if (wsi->mux_substream)
 					lwsi_set_state(wsi, LRS_ESTABLISHED);
 			}
 
@@ -223,7 +230,7 @@ postbody_completion:
 	case LRS_SHUTDOWN:
 
 ws_mode:
-#if !defined(LWS_NO_CLIENT) && defined(LWS_ROLE_WS)
+#if defined(LWS_WITH_CLIENT) && defined(LWS_ROLE_WS)
 		// lwsl_notice("%s: ws_mode\n", __func__);
 		if (lws_ws_handshake_client(wsi, &buf, (size_t)len))
 			goto bail;
@@ -249,6 +256,9 @@ ws_mode:
 	case LRS_SSL_ACK_PENDING:
 		break;
 
+	case LRS_FLUSHING_BEFORE_CLOSE:
+		break;
+
 	case LRS_DEAD_SOCKET:
 		lwsl_err("%s: Unhandled state LRS_DEAD_SOCKET\n", __func__);
 		goto bail;
@@ -287,7 +297,7 @@ bail:
 
 	return -1;
 }
-#if !defined(LWS_NO_SERVER)
+#if defined(LWS_WITH_SERVER)
 static int
 lws_h1_server_socket_service(struct lws *wsi, struct lws_pollfd *pollfd)
 {
@@ -349,7 +359,9 @@ lws_h1_server_socket_service(struct lws *wsi, struct lws_pollfd *pollfd)
 		 * exhausted and we tried to do a read of some kind.
 		 */
 
-		buffered = lws_buflist_aware_read(pt, wsi, &ebuf);
+		ebuf.token = NULL;
+		ebuf.len = 0;
+		buffered = lws_buflist_aware_read(pt, wsi, &ebuf, 0, __func__);
 		switch (ebuf.len) {
 		case 0:
 			lwsl_info("%s: read 0 len a\n", __func__);
@@ -386,7 +398,8 @@ lws_h1_server_socket_service(struct lws *wsi, struct lws_pollfd *pollfd)
 
 		if (lwsi_state(wsi) == LRS_ISSUING_FILE) {
 			// lwsl_notice("stashing: wsi %p: bd %d\n", wsi, buffered);
-			if (lws_buflist_aware_consume(wsi, &ebuf, 0, buffered))
+			if (lws_buflist_aware_finished_consuming(wsi, &ebuf, 0,
+							buffered, __func__))
 				return LWS_HPI_RET_PLEASE_CLOSE_ME;
 
 			goto try_pollout;
@@ -405,9 +418,10 @@ lws_h1_server_socket_service(struct lws *wsi, struct lws_pollfd *pollfd)
 		if (n < 0) /* we closed wsi */
 			return LWS_HPI_RET_WSI_ALREADY_DIED;
 
-		lwsl_debug("%s: consumed %d\n", __func__, n);
+		// lwsl_notice("%s: consumed %d\n", __func__, n);
 
-		if (lws_buflist_aware_consume(wsi, &ebuf, n, buffered))
+		if (lws_buflist_aware_finished_consuming(wsi, &ebuf, n,
+							 buffered, __func__))
 			return LWS_HPI_RET_PLEASE_CLOSE_ME;
 
 		/*
@@ -504,6 +518,8 @@ try_pollout:
 		return LWS_HPI_RET_HANDLED;
 	}
 
+#if defined(LWS_WITH_FILE_OPS)
+
 	/* >0 == completion, <0 == error
 	 *
 	 * We'll get a LWS_CALLBACK_HTTP_FILE_COMPLETION callback when
@@ -513,6 +529,7 @@ try_pollout:
 	n = lws_serve_http_file_fragment(wsi);
 	if (n < 0)
 		goto fail;
+#endif
 
 	return LWS_HPI_RET_HANDLED;
 
@@ -615,12 +632,12 @@ rops_handle_POLLIN_h1(struct lws_context_per_thread *pt, struct lws *wsi,
                  */
 		return LWS_HPI_RET_HANDLED;
 
-#if !defined(LWS_NO_SERVER)
+#if defined(LWS_WITH_SERVER)
 	if (!lwsi_role_client(wsi)) {
 		int n;
 
 		lwsl_debug("%s: %p: wsistate 0x%x\n", __func__, wsi,
-			   wsi->wsistate);
+			   (int)wsi->wsistate);
 		n = lws_h1_server_socket_service(wsi, pollfd);
 		if (n != LWS_HPI_RET_HANDLED)
 			return n;
@@ -633,7 +650,7 @@ rops_handle_POLLIN_h1(struct lws_context_per_thread *pt, struct lws *wsi,
 	}
 #endif
 
-#ifndef LWS_NO_CLIENT
+#if defined(LWS_WITH_CLIENT)
 	if ((pollfd->revents & LWS_POLLIN) &&
 	     wsi->hdr_parsing_completed && !wsi->told_user_closed) {
 
@@ -671,14 +688,14 @@ rops_handle_POLLIN_h1(struct lws_context_per_thread *pt, struct lws *wsi,
 //	if (lwsi_state(wsi) == LRS_ESTABLISHED)
 //		return LWS_HPI_RET_HANDLED;
 
-#if !defined(LWS_NO_CLIENT)
+#if defined(LWS_WITH_CLIENT)
 	if ((pollfd->revents & LWS_POLLOUT) &&
 	    lws_handle_POLLOUT_event(wsi, pollfd)) {
 		lwsl_debug("POLLOUT event closed it\n");
 		return LWS_HPI_RET_PLEASE_CLOSE_ME;
 	}
 
-	if (lws_client_socket_service(wsi, pollfd, NULL))
+	if (lws_client_socket_service(wsi, pollfd))
 		return LWS_HPI_RET_WSI_ALREADY_DIED;
 #endif
 
@@ -758,7 +775,7 @@ rops_write_role_protocol_h1(struct lws *wsi, unsigned char *buf, size_t len,
 #if defined(LWS_WITH_HTTP_STREAM_COMPRESSION)
 	if (wsi->http.lcs && (((*wp) & 0x1f) == LWS_WRITE_HTTP_FINAL ||
 			      ((*wp) & 0x1f) == LWS_WRITE_HTTP)) {
-		unsigned char mtubuf[1400 + LWS_PRE +
+		unsigned char mtubuf[1500 + LWS_PRE +
 				     LWS_HTTP_CHUNK_HDR_MAX_SIZE +
 				     LWS_HTTP_CHUNK_TRL_MAX_SIZE],
 			      *out = mtubuf + LWS_PRE +
@@ -820,7 +837,7 @@ static int
 rops_alpn_negotiated_h1(struct lws *wsi, const char *alpn)
 {
 	lwsl_debug("%s: client %d\n", __func__, lwsi_role_client(wsi));
-#if !defined(LWS_NO_CLIENT)
+#if defined(LWS_WITH_CLIENT)
 	if (lwsi_role_client(wsi)) {
 		/*
 		 * If alpn asserts it is http/1.1, server support for KA is
@@ -869,7 +886,7 @@ rops_destroy_role_h1(struct lws *wsi)
 	return 0;
 }
 
-#if !defined(LWS_NO_SERVER)
+#if defined(LWS_WITH_SERVER)
 
 static int
 rops_adoption_bind_h1(struct lws *wsi, int type, const char *vh_prot_name)
@@ -914,10 +931,10 @@ rops_adoption_bind_h1(struct lws *wsi, int type, const char *vh_prot_name)
 
 #endif
 
-#if !defined(LWS_NO_CLIENT)
+#if defined(LWS_WITH_CLIENT)
 
 static const char * const http_methods[] = {
-	"GET", "POST", "OPTIONS", "PUT", "PATCH", "DELETE", "CONNECT"
+	"GET", "POST", "OPTIONS", "HEAD", "PUT", "PATCH", "DELETE", "CONNECT"
 };
 
 static int
@@ -934,7 +951,7 @@ rops_client_bind_h1(struct lws *wsi, const struct lws_client_connect_info *i)
 		 * we can assign the user space now, otherwise do it after the
 		 * ws subprotocol negotiated
 		 */
-		if (!wsi->user_space && wsi->stash->method)
+		if (!wsi->user_space && wsi->stash->cis[CIS_METHOD])
 			if (lws_ensure_user_space(wsi))
 				return 1;
 
@@ -948,11 +965,8 @@ rops_client_bind_h1(struct lws *wsi, const struct lws_client_connect_info *i)
 		  * only try h2 if he assertively said to use h2 alpn, otherwise
 		  * ws implies alpn restriction to h1.
 		  */
-		if (!wsi->stash->method && !wsi->stash->alpn) {
-			wsi->stash->alpn = lws_strdup("http/1.1");
-			if (!wsi->stash->alpn)
-				return 1;
-		}
+		if (!wsi->stash->cis[CIS_METHOD] && !wsi->stash->cis[CIS_ALPN])
+			wsi->stash->cis[CIS_ALPN] = "http/1.1";
 
 		/* if we went on the ah waiting list, it's ok, we can wait.
 		 *
@@ -960,7 +974,7 @@ rops_client_bind_h1(struct lws *wsi, const struct lws_client_connect_info *i)
 		 * lws_http_client_connect_via_info2().
 		 */
 		if (lws_header_table_attach(wsi, 0)
-#ifndef LWS_NO_CLIENT
+#if defined(LWS_WITH_CLIENT)
 				< 0)
 			/*
 			 * if we failed here, the connection is already closed
@@ -977,9 +991,13 @@ rops_client_bind_h1(struct lws *wsi, const struct lws_client_connect_info *i)
 
 	/*
 	 * Clients that want to be h1, h2, or ws all start out as h1
-	 * (we don't yet know if the server supports h2 or ws)
+	 * (we don't yet know if the server supports h2 or ws), unless their
+	 * alpn is only "h2"
 	 */
 
+//	if (i->alpn && !strcmp(i->alpn, "h2"))
+//		return 0; /* we are h1, he only wants h2 */
+
 	if (!i->method) { /* websockets */
 #if defined(LWS_ROLE_WS)
 		if (lws_create_client_ws_object(i, wsi))
@@ -1084,53 +1102,49 @@ static int
 rops_close_kill_connection_h1(struct lws *wsi, enum lws_close_status reason)
 {
 #if defined(LWS_WITH_HTTP_PROXY)
-	struct lws *wsi_eff = lws_client_wsi_effective(wsi);
-
-	if (!wsi_eff->http.proxy_clientside)
+	if (!wsi->http.proxy_clientside)
 		return 0;
 
-	wsi_eff->http.proxy_clientside = 0;
+	wsi->http.proxy_clientside = 0;
 
-	if (user_callback_handle_rxflow(wsi_eff->protocol->callback, wsi_eff,
+	if (user_callback_handle_rxflow(wsi->protocol->callback, wsi,
 					LWS_CALLBACK_COMPLETED_CLIENT_HTTP,
-					wsi_eff->user_space, NULL, 0))
+					wsi->user_space, NULL, 0))
 		return 0;
 #endif
 	return 0;
 }
 
 int
-rops_init_context_h1(struct lws_context *context,
-		     const struct lws_context_creation_info *info)
+rops_pt_init_destroy_h1(struct lws_context *context,
+		    const struct lws_context_creation_info *info,
+		    struct lws_context_per_thread *pt, int destroy)
 {
 	/*
 	 * We only want to do this once... we will do it if no h2 support
 	 * otherwise let h2 ops do it.
 	 */
-#if !defined(LWS_ROLE_H2)
-	int n;
-
-	for (n = 0; n < context->count_threads; n++) {
-		struct lws_context_per_thread *pt = &context->pt[n];
+#if !defined(LWS_ROLE_H2) && defined(LWS_WITH_SERVER)
+	if (!destroy) {
 
 		pt->sul_ah_lifecheck.cb = lws_sul_http_ah_lifecheck;
 
 		__lws_sul_insert(&pt->pt_sul_owner, &pt->sul_ah_lifecheck,
 				 30 * LWS_US_PER_SEC);
-	}
+	} else
+		lws_dll2_remove(&pt->sul_ah_lifecheck.list);
 #endif
 
 	return 0;
 }
 
-struct lws_role_ops role_ops_h1 = {
+const struct lws_role_ops role_ops_h1 = {
 	/* role name */			"h1",
 	/* alpn id */			"http/1.1",
 	/* check_upgrades */		NULL,
-	/* init_context */		rops_init_context_h1,
+	/* pt_init_destroy */		rops_pt_init_destroy_h1,
 	/* init_vhost */		NULL,
 	/* destroy_vhost */		NULL,
-	/* periodic_checks */		NULL,
 	/* service_flag_pending */	NULL,
 	/* handle_POLLIN */		rops_handle_POLLIN_h1,
 	/* handle_POLLOUT */		rops_handle_POLLOUT_h1,
@@ -1144,16 +1158,17 @@ struct lws_role_ops role_ops_h1 = {
 	/* close_role */		NULL,
 	/* close_kill_connection */	rops_close_kill_connection_h1,
 	/* destroy_role */		rops_destroy_role_h1,
-#if !defined(LWS_NO_SERVER)
+#if defined(LWS_WITH_SERVER)
 	/* adoption_bind */		rops_adoption_bind_h1,
 #else
 					NULL,
 #endif
-#if !defined(LWS_NO_CLIENT)
+#if defined(LWS_WITH_CLIENT)
 	/* client_bind */		rops_client_bind_h1,
 #else
 					NULL,
 #endif
+	/* issue_keepalive */		NULL,
 	/* adoption_cb clnt, srv */	{ LWS_CALLBACK_SERVER_NEW_CLIENT_INSTANTIATED,
 					  LWS_CALLBACK_SERVER_NEW_CLIENT_INSTANTIATED },
 	/* rx_cb clnt, srv */		{ LWS_CALLBACK_RECEIVE_CLIENT_HTTP,
