@@ -1,28 +1,31 @@
-/*
- * libwebsockets - generic EC api hiding the backend - mbedtls implementation
+ /*
+ * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2017 - 2018 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2019 Andy Green <andy@warmcat.com>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation:
- *  version 2.1 of the License.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA  02110-1301  USA
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
  *
  *  lws_genec provides an EC abstraction api in lws that works the
  *  same whether you are using openssl or mbedtls crypto functions underneath.
  */
-#include "core/private.h"
-#include "tls/mbedtls/private.h"
+#include "private-lib-core.h"
+#include "private-lib-tls-mbedtls.h"
 
 const struct lws_ec_curves lws_ec_curves[] = {
 	/*
@@ -135,7 +138,7 @@ bail1:
 	return ret;
 }
 
-LWS_VISIBLE int
+int
 lws_genecdh_create(struct lws_genec_ctx *ctx, struct lws_context *context,
 		   const struct lws_ec_curves *curve_table)
 {
@@ -154,7 +157,7 @@ lws_genecdh_create(struct lws_genec_ctx *ctx, struct lws_context *context,
 	return 0;
 }
 
-LWS_VISIBLE int
+int
 lws_genecdsa_create(struct lws_genec_ctx *ctx, struct lws_context *context,
 		    const struct lws_ec_curves *curve_table)
 {
@@ -174,7 +177,7 @@ lws_genecdsa_create(struct lws_genec_ctx *ctx, struct lws_context *context,
 }
 
 
-LWS_VISIBLE int
+int
 lws_genecdh_set_key(struct lws_genec_ctx *ctx, struct lws_gencrypto_keyelem *el,
 		    enum enum_lws_dh_side side)
 {
@@ -184,7 +187,7 @@ lws_genecdh_set_key(struct lws_genec_ctx *ctx, struct lws_gencrypto_keyelem *el,
 	return lws_genec_keypair_import(ctx, side, el);
 }
 
-LWS_VISIBLE int
+int
 lws_genecdsa_set_key(struct lws_genec_ctx *ctx,
 		     struct lws_gencrypto_keyelem *el)
 {
@@ -194,7 +197,7 @@ lws_genecdsa_set_key(struct lws_genec_ctx *ctx,
 	return lws_genec_keypair_import(ctx, 0, el);
 }
 
-LWS_VISIBLE void
+void
 lws_genec_destroy(struct lws_genec_ctx *ctx)
 {
 	switch (ctx->genec_alg) {
@@ -217,7 +220,7 @@ lws_genec_destroy(struct lws_genec_ctx *ctx)
 	}
 }
 
-LWS_VISIBLE int
+int
 lws_genecdh_new_keypair(struct lws_genec_ctx *ctx, enum enum_lws_dh_side side,
 			const char *curve_name,
 			struct lws_gencrypto_keyelem *el)
@@ -301,7 +304,7 @@ bail1:
 	return -1;
 }
 
-LWS_VISIBLE int
+int
 lws_genecdsa_new_keypair(struct lws_genec_ctx *ctx, const char *curve_name,
 			 struct lws_gencrypto_keyelem *el)
 {
@@ -374,7 +377,7 @@ bail1:
 	return -1;
 }
 
-LWS_VISIBLE LWS_EXTERN int
+int
 lws_genecdsa_hash_sign_jws(struct lws_genec_ctx *ctx, const uint8_t *in,
 			   enum lws_genhash_types hash_type, int keybits,
 			   uint8_t *sig, size_t sig_len)
@@ -436,7 +439,7 @@ bail1:
 	return -3;
 }
 
-LWS_VISIBLE LWS_EXTERN int
+int
 lws_genecdsa_hash_sig_verify_jws(struct lws_genec_ctx *ctx, const uint8_t *in,
 				 enum lws_genhash_types hash_type, int keybits,
 				 const uint8_t *sig, size_t sig_len)
