@@ -18,14 +18,18 @@
 
 /* mbedtls include */
 #include "mbedtls/platform.h"
+#if defined(LWS_HAVE_MBEDTLS_NET_SOCKETS)
 #include "mbedtls/net_sockets.h"
+#else
+#include "mbedtls/net.h"
+#endif
 #include "mbedtls/debug.h"
 #include "mbedtls/entropy.h"
 #include "mbedtls/ctr_drbg.h"
 #include "mbedtls/error.h"
 #include "mbedtls/certs.h"
 
-#include "core/private.h"
+#include "private-lib-core.h"
 
 #define X509_INFO_STRING_LENGTH 8192
 
