@@ -173,7 +173,7 @@ static const struct lws_protocols protocols[] = {
 	},
 };
 
-LWS_EXTERN LWS_VISIBLE int
+LWS_VISIBLE int
 init_protocol_client_loopback_test(struct lws_context *context,
 				   struct lws_plugin_capability *c)
 {
@@ -191,7 +191,7 @@ init_protocol_client_loopback_test(struct lws_context *context,
 	return 0;
 }
 
-LWS_EXTERN LWS_VISIBLE int
+LWS_VISIBLE int
 destroy_protocol_client_loopback_test(struct lws_context *context)
 {
 	return 0;
