@@ -121,7 +121,7 @@ static const struct lws_protocols protocols[] = {
 	LWS_PLUGIN_PROTOCOL_DUMB_INCREMENT
 };
 
-LWS_EXTERN LWS_VISIBLE int
+LWS_VISIBLE int
 init_protocol_dumb_increment(struct lws_context *context,
 			     struct lws_plugin_capability *c)
 {
@@ -139,7 +139,7 @@ init_protocol_dumb_increment(struct lws_context *context,
 	return 0;
 }
 
-LWS_EXTERN LWS_VISIBLE int
+LWS_VISIBLE int
 destroy_protocol_dumb_increment(struct lws_context *context)
 {
 	return 0;
