@@ -278,7 +278,7 @@ static const struct lws_protocols protocols[] = {
 	LWS_PLUGIN_PROTOCOL_RAW_TEST
 };
 
-LWS_EXTERN LWS_VISIBLE int
+LWS_VISIBLE int
 init_protocol_lws_raw_test(struct lws_context *context,
 			     struct lws_plugin_capability *c)
 {
@@ -296,7 +296,7 @@ init_protocol_lws_raw_test(struct lws_context *context,
 	return 0;
 }
 
-LWS_EXTERN LWS_VISIBLE int
+LWS_VISIBLE int
 destroy_protocol_lws_raw_test(struct lws_context *context)
 {
 	return 0;
